@@ -3,7 +3,6 @@ package consul
 import (
 	"fmt"
 	"io"
-	"log"
 	"os"
 	"strconv"
 	"sync"
@@ -15,7 +14,9 @@ import (
 	"github.com/hashicorp/consul/agent/router"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/tlsutil"
+	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/serf/serf"
 	"golang.org/x/time/rate"
 )
@@ -72,7 +73,7 @@ type Client struct {
 	eventCh chan serf.Event
 
 	// Logger uses the provided LogOutput
-	logger *log.Logger
+	logger hclog.InterceptLogger
 
 	// serf is the Serf cluster maintained inside the DC
 	// which contains all the DC nodes
@@ -100,7 +101,7 @@ func NewClient(config *Config) (*Client, error) {
 	return NewClientLogger(config, nil, c)
 }
 
-func NewClientLogger(config *Config, logger *log.Logger, tlsConfigurator *tlsutil.Configurator) (*Client, error) {
+func NewClientLogger(config *Config, logger hclog.InterceptLogger, tlsConfigurator *tlsutil.Configurator) (*Client, error) {
 	// Check the protocol version
 	if err := config.CheckProtocolVersion(); err != nil {
 		return nil, err
@@ -123,7 +124,10 @@ func NewClientLogger(config *Config, logger *log.Logger, tlsConfigurator *tlsuti
 
 	// Create a logger
 	if logger == nil {
-		logger = log.New(config.LogOutput, "", log.LstdFlags)
+		logger = hclog.NewInterceptLogger(&hclog.LoggerOptions{
+			Level:  hclog.Debug,
+			Output: config.LogOutput,
+		})
 	}
 
 	connPool := &pool.ConnPool{
@@ -140,7 +144,7 @@ func NewClientLogger(config *Config, logger *log.Logger, tlsConfigurator *tlsuti
 		config:          config,
 		connPool:        connPool,
 		eventCh:         make(chan serf.Event, serfEventBacklog),
-		logger:          logger,
+		logger:          logger.NamedIntercept(logging.ConsulClient),
 		shutdownCh:      make(chan struct{}),
 		tlsConfigurator: tlsConfigurator,
 	}
@@ -156,10 +160,10 @@ func NewClientLogger(config *Config, logger *log.Logger, tlsConfigurator *tlsuti
 	aclConfig := ACLResolverConfig{
 		Config:      config,
 		Delegate:    c,
-		Logger:      logger,
+		Logger:      c.logger,
 		AutoDisable: true,
 		CacheConfig: clientACLCacheConfig,
-		Sentinel:    nil,
+		ACLConfig:   newACLConfig(c.logger),
 	}
 	var err error
 	if c.acls, err = NewACLResolver(&aclConfig); err != nil {
@@ -197,7 +201,7 @@ func NewClientLogger(config *Config, logger *log.Logger, tlsConfigurator *tlsuti
 
 // Shutdown is used to shutdown the client
 func (c *Client) Shutdown() error {
-	c.logger.Printf("[INFO] consul: shutting down client")
+	c.logger.Info("shutting down client")
 	c.shutdownLock.Lock()
 	defer c.shutdownLock.Unlock()
 
@@ -214,17 +218,20 @@ func (c *Client) Shutdown() error {
 
 	// Close the connection pool
 	c.connPool.Shutdown()
+
+	c.acls.Close()
+
 	return nil
 }
 
 // Leave is used to prepare for a graceful shutdown
 func (c *Client) Leave() error {
-	c.logger.Printf("[INFO] consul: client starting leave")
+	c.logger.Info("client starting leave")
 
 	// Leave the LAN pool
 	if c.serf != nil {
 		if err := c.serf.Leave(); err != nil {
-			c.logger.Printf("[ERR] consul: Failed to leave LAN Serf cluster: %v", err)
+			c.logger.Error("Failed to leave LAN Serf cluster", "error", err)
 		}
 	}
 	return nil
@@ -263,7 +270,10 @@ func (c *Client) LANSegmentMembers(segment string) ([]serf.Member, error) {
 }
 
 // RemoveFailedNode is used to remove a failed node from the cluster
-func (c *Client) RemoveFailedNode(node string) error {
+func (c *Client) RemoveFailedNode(node string, prune bool) error {
+	if prune {
+		return c.serf.RemoveFailedNodePrune(node)
+	}
 	return c.serf.RemoveFailedNode(node)
 }
 
@@ -307,7 +317,11 @@ TRY:
 	}
 
 	// Move off to another server, and see if we can retry.
-	c.logger.Printf("[ERR] consul: %q RPC failed to server %s: %v", method, server.Addr, rpcErr)
+	c.logger.Error("RPC failed to server",
+		"method", method,
+		"server", server.Addr,
+		"error", rpcErr,
+	)
 	metrics.IncrCounterWithLabels([]string{"client", "rpc", "failed"}, 1, []metrics.Label{{Name: "server", Value: server.Name}})
 	c.routers.NotifyFailedServer(server)
 	if retry := canRetry(args, rpcErr); !retry {
@@ -351,7 +365,7 @@ func (c *Client) SnapshotRPC(args *structs.SnapshotRequest, in io.Reader, out io
 	}
 	defer func() {
 		if err := snap.Close(); err != nil {
-			c.logger.Printf("[WARN] consul: Failed closing snapshot stream: %v", err)
+			c.logger.Error("Failed closing snapshot stream", "error", err)
 		}
 	}()
 
