@@ -2,6 +2,7 @@ package consul
 
 import (
 	"fmt"
+	"strings"
 	"time"
 
 	"github.com/armon/go-metrics"
@@ -9,30 +10,33 @@ import (
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/sentinel"
+	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
 )
 
 // KVS endpoint is used to manipulate the Key-Value store
 type KVS struct {
-	srv *Server
+	srv    *Server
+	logger hclog.Logger
 }
 
 // preApply does all the verification of a KVS update that is performed BEFORE
 // we submit as a Raft log entry. This includes enforcing the lock delay which
 // must only be done on the leader.
-func kvsPreApply(srv *Server, rule acl.Authorizer, op api.KVOp, dirEnt *structs.DirEntry) (bool, error) {
+func kvsPreApply(logger hclog.Logger, srv *Server, authz acl.Authorizer, op api.KVOp, dirEnt *structs.DirEntry) (bool, error) {
 	// Verify the entry.
-
 	if dirEnt.Key == "" && op != api.KVDeleteTree {
 		return false, fmt.Errorf("Must provide key")
 	}
 
 	// Apply the ACL policy if any.
-	if rule != nil {
+	if authz != nil {
 		switch op {
 		case api.KVDeleteTree:
-			if !rule.KeyWritePrefix(dirEnt.Key) {
+			var authzContext acl.AuthorizerContext
+			dirEnt.FillAuthzContext(&authzContext)
+
+			if authz.KeyWritePrefix(dirEnt.Key, &authzContext) != acl.Allow {
 				return false, acl.ErrPermissionDenied
 			}
 
@@ -43,15 +47,18 @@ func kvsPreApply(srv *Server, rule acl.Authorizer, op api.KVOp, dirEnt *structs.
 			// These could reveal information based on the outcome
 			// of the transaction, and they operate on individual
 			// keys so we check them here.
-			if !rule.KeyRead(dirEnt.Key) {
+			var authzContext acl.AuthorizerContext
+			dirEnt.FillAuthzContext(&authzContext)
+
+			if authz.KeyRead(dirEnt.Key, &authzContext) != acl.Allow {
 				return false, acl.ErrPermissionDenied
 			}
 
 		default:
-			scope := func() map[string]interface{} {
-				return sentinel.ScopeKVUpsert(dirEnt.Key, dirEnt.Value, dirEnt.Flags)
-			}
-			if !rule.KeyWrite(dirEnt.Key, scope) {
+			var authzContext acl.AuthorizerContext
+			dirEnt.FillAuthzContext(&authzContext)
+
+			if authz.KeyWrite(dirEnt.Key, &authzContext) != acl.Allow {
 				return false, acl.ErrPermissionDenied
 			}
 		}
@@ -65,10 +72,12 @@ func kvsPreApply(srv *Server, rule acl.Authorizer, op api.KVOp, dirEnt *structs.
 	// only the wall-time of the leader node is used, preventing any inconsistencies.
 	if op == api.KVLock {
 		state := srv.fsm.State()
-		expires := state.KVSLockDelay(dirEnt.Key)
+		expires := state.KVSLockDelay(dirEnt.Key, &dirEnt.EnterpriseMeta)
 		if expires.After(time.Now()) {
-			srv.logger.Printf("[WARN] consul.kvs: Rejecting lock of %s due to lock-delay until %v",
-				dirEnt.Key, expires)
+			logger.Warn("Rejecting lock of key due to lock-delay",
+				"key", dirEnt.Key,
+				"expire_time", expires.String(),
+			)
 			return false, nil
 		}
 	}
@@ -84,11 +93,16 @@ func (k *KVS) Apply(args *structs.KVSRequest, reply *bool) error {
 	defer metrics.MeasureSince([]string{"kvs", "apply"}, time.Now())
 
 	// Perform the pre-apply checks.
-	acl, err := k.srv.ResolveToken(args.Token)
+	authz, err := k.srv.ResolveTokenAndDefaultMeta(args.Token, &args.DirEnt.EnterpriseMeta, nil)
 	if err != nil {
 		return err
 	}
-	ok, err := kvsPreApply(k.srv, acl, args.Op, &args.DirEnt)
+
+	if err := k.srv.validateEnterpriseRequest(&args.DirEnt.EnterpriseMeta, true); err != nil {
+		return err
+	}
+
+	ok, err := kvsPreApply(k.logger, k.srv, authz, args.Op, &args.DirEnt)
 	if err != nil {
 		return err
 	}
@@ -100,7 +114,7 @@ func (k *KVS) Apply(args *structs.KVSRequest, reply *bool) error {
 	// Apply the update.
 	resp, err := k.srv.raftApply(structs.KVSRequestType, args)
 	if err != nil {
-		k.srv.logger.Printf("[ERR] consul.kvs: Apply failed: %v", err)
+		k.logger.Error("Raft apply failed", "error", err)
 		return err
 	}
 	if respErr, ok := resp.(error); ok {
@@ -120,19 +134,25 @@ func (k *KVS) Get(args *structs.KeyRequest, reply *structs.IndexedDirEntries) er
 		return err
 	}
 
-	aclRule, err := k.srv.ResolveToken(args.Token)
+	var authzContext acl.AuthorizerContext
+	authz, err := k.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
 	if err != nil {
 		return err
 	}
+
+	if err := k.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return k.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, ent, err := state.KVSGet(ws, args.Key)
+			index, ent, err := state.KVSGet(ws, args.Key, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
-			if aclRule != nil && !aclRule.KeyRead(args.Key) {
+			if authz != nil && authz.KeyRead(args.Key, &authzContext) != acl.Allow {
 				return acl.ErrPermissionDenied
 			}
 
@@ -159,12 +179,17 @@ func (k *KVS) List(args *structs.KeyRequest, reply *structs.IndexedDirEntries) e
 		return err
 	}
 
-	aclToken, err := k.srv.ResolveToken(args.Token)
+	var authzContext acl.AuthorizerContext
+	authz, err := k.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
 	if err != nil {
 		return err
 	}
 
-	if aclToken != nil && k.srv.config.ACLEnableKeyListPolicy && !aclToken.KeyList(args.Key) {
+	if err := k.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
+	if authz != nil && k.srv.config.ACLEnableKeyListPolicy && authz.KeyList(args.Key, &authzContext) != acl.Allow {
 		return acl.ErrPermissionDenied
 	}
 
@@ -172,12 +197,12 @@ func (k *KVS) List(args *structs.KeyRequest, reply *structs.IndexedDirEntries) e
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, ent, err := state.KVSList(ws, args.Key)
+			index, ent, err := state.KVSList(ws, args.Key, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
-			if aclToken != nil {
-				ent = FilterDirEnt(aclToken, ent)
+			if authz != nil {
+				ent = FilterDirEnt(authz, ent)
 			}
 
 			if len(ent) == 0 {
@@ -198,17 +223,25 @@ func (k *KVS) List(args *structs.KeyRequest, reply *structs.IndexedDirEntries) e
 }
 
 // ListKeys is used to list all keys with a given prefix to a separator.
+// An optional separator may be specified, which can be used to slice off a part
+// of the response so that only a subset of the prefix is returned. In this
+// mode, the keys which are omitted are still counted in the returned index.
 func (k *KVS) ListKeys(args *structs.KeyListRequest, reply *structs.IndexedKeyList) error {
 	if done, err := k.srv.forward("KVS.ListKeys", args, args, reply); done {
 		return err
 	}
 
-	aclToken, err := k.srv.ResolveToken(args.Token)
+	var authzContext acl.AuthorizerContext
+	authz, err := k.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
 	if err != nil {
 		return err
 	}
 
-	if aclToken != nil && k.srv.config.ACLEnableKeyListPolicy && !aclToken.KeyList(args.Prefix) {
+	if err := k.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
+	if authz != nil && k.srv.config.ACLEnableKeyListPolicy && authz.KeyList(args.Prefix, &authzContext) != acl.Allow {
 		return acl.ErrPermissionDenied
 	}
 
@@ -216,7 +249,7 @@ func (k *KVS) ListKeys(args *structs.KeyListRequest, reply *structs.IndexedKeyLi
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, keys, err := state.KVSListKeys(ws, args.Prefix, args.Seperator)
+			index, entries, err := state.KVSList(ws, args.Prefix, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
@@ -229,8 +262,37 @@ func (k *KVS) ListKeys(args *structs.KeyListRequest, reply *structs.IndexedKeyLi
 				reply.Index = index
 			}
 
-			if aclToken != nil {
-				keys = FilterKeys(aclToken, keys)
+			if authz != nil {
+				entries = FilterDirEnt(authz, entries)
+			}
+
+			// Collect the keys from the filtered entries
+			prefixLen := len(args.Prefix)
+			sepLen := len(args.Seperator)
+
+			var keys []string
+			seen := make(map[string]bool)
+
+			for _, e := range entries {
+				// Always accumulate if no separator provided
+				if sepLen == 0 {
+					keys = append(keys, e.Key)
+					continue
+				}
+
+				// Parse and de-duplicate the returned keys based on the
+				// key separator, if provided.
+				after := e.Key[prefixLen:]
+				sepIdx := strings.Index(after, args.Seperator)
+				if sepIdx > -1 {
+					key := e.Key[:prefixLen+sepIdx+sepLen]
+					if ok := seen[key]; !ok {
+						keys = append(keys, key)
+						seen[key] = true
+					}
+				} else {
+					keys = append(keys, e.Key)
+				}
 			}
 			reply.Keys = keys
 			return nil
