@@ -2,10 +2,12 @@ package consul
 
 import (
 	"crypto/tls"
+	"errors"
 	"fmt"
 	"io"
 	"net"
 	"strings"
+	"sync/atomic"
 	"time"
 
 	"github.com/armon/go-metrics"
@@ -14,20 +16,18 @@ import (
 	"github.com/hashicorp/consul/agent/pool"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/logging"
+	connlimit "github.com/hashicorp/go-connlimit"
+	"github.com/hashicorp/go-hclog"
 	memdb "github.com/hashicorp/go-memdb"
+	"github.com/hashicorp/go-raftchunking"
 	"github.com/hashicorp/memberlist"
 	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
+	"github.com/hashicorp/raft"
 	"github.com/hashicorp/yamux"
 )
 
 const (
-	// maxQueryTime is used to bound the limit of a blocking query
-	maxQueryTime = 600 * time.Second
-
-	// defaultQueryTime is the amount of time we block waiting for a change
-	// if no time is specified. Previously we would wait the maxQueryTime.
-	defaultQueryTime = 300 * time.Second
-
 	// jitterFraction is a the limit to the amount of jitter we apply
 	// to a user specified MaxQueryTime. We divide the specified time by
 	// the fraction. So 16 == 6.25% limit of jitter. This same fraction
@@ -45,6 +45,14 @@ const (
 	enqueueLimit = 30 * time.Second
 )
 
+var (
+	ErrChunkingResubmit = errors.New("please resubmit call for rechunking")
+)
+
+func (s *Server) rpcLogger() hclog.Logger {
+	return s.loggers.Named(logging.RPC)
+}
+
 // listen is used to listen for incoming RPC connections
 func (s *Server) listen(listener net.Listener) {
 	for {
@@ -54,10 +62,20 @@ func (s *Server) listen(listener net.Listener) {
 			if s.shutdown {
 				return
 			}
-			s.logger.Printf("[ERR] consul.rpc: failed to accept RPC conn: %v", err)
+			s.rpcLogger().Error("failed to accept RPC conn", "error", err)
 			continue
 		}
 
+		free, err := s.rpcConnLimiter.Accept(conn)
+		if err != nil {
+			s.rpcLogger().Error("rejecting RPC conn from %s"+
+				" rpc_max_conns_per_client exceeded", conn.RemoteAddr().String())
+			conn.Close()
+			continue
+		}
+		// Wrap conn so it will be auto-freed from conn limiter when it closes.
+		conn = connlimit.Wrap(conn, free)
+
 		go s.handleConn(conn, false)
 		metrics.IncrCounter([]string{"rpc", "accept_conn"}, 1)
 	}
@@ -72,20 +90,38 @@ func logConn(conn net.Conn) string {
 // handleConn is used to determine if this is a Raft or
 // Consul type RPC connection and invoke the correct handler
 func (s *Server) handleConn(conn net.Conn, isTLS bool) {
+
 	// Read a single byte
 	buf := make([]byte, 1)
+
+	// Limit how long the client can hold the connection open before they send the
+	// magic byte (and authenticate when mTLS is enabled). If `isTLS == true` then
+	// this also enforces a timeout on how long it takes for the handshake to
+	// complete since tls.Conn.Read implicitly calls Handshake().
+	if s.config.RPCHandshakeTimeout > 0 {
+		conn.SetReadDeadline(time.Now().Add(s.config.RPCHandshakeTimeout))
+	}
 	if _, err := conn.Read(buf); err != nil {
 		if err != io.EOF {
-			s.logger.Printf("[ERR] consul.rpc: failed to read byte: %v %s", err, logConn(conn))
+			s.rpcLogger().Error("failed to read byte",
+				"conn", logConn(conn),
+				"error", err,
+			)
 		}
 		conn.Close()
 		return
 	}
 	typ := pool.RPCType(buf[0])
 
+	// Reset the deadline as we aren't sure what is expected next - it depends on
+	// the protocol.
+	if s.config.RPCHandshakeTimeout > 0 {
+		conn.SetReadDeadline(time.Time{})
+	}
+
 	// Enforce TLS if VerifyIncoming is set
 	if s.tlsConfigurator.VerifyIncomingRPC() && !isTLS && typ != pool.RPCTLS && typ != pool.RPCTLSInsecure {
-		s.logger.Printf("[WARN] consul.rpc: Non-TLS connection attempted with VerifyIncoming set %s", logConn(conn))
+		s.rpcLogger().Warn("Non-TLS connection attempted with VerifyIncoming set", "conn", logConn(conn))
 		conn.Close()
 		return
 	}
@@ -100,6 +136,12 @@ func (s *Server) handleConn(conn net.Conn, isTLS bool) {
 		s.raftLayer.Handoff(conn)
 
 	case pool.RPCTLS:
+		// Don't allow malicious client to create TLS-in-TLS for ever.
+		if isTLS {
+			s.rpcLogger().Error("TLS connection attempting to establish inner TLS connection %s", logConn(conn))
+			conn.Close()
+			return
+		}
 		conn = tls.Server(conn, s.tlsConfigurator.IncomingRPCConfig())
 		s.handleConn(conn, true)
 
@@ -110,12 +152,21 @@ func (s *Server) handleConn(conn net.Conn, isTLS bool) {
 		s.handleSnapshotConn(conn)
 
 	case pool.RPCTLSInsecure:
+		// Don't allow malicious client to create TLS-in-TLS for ever.
+		if isTLS {
+			s.rpcLogger().Error("TLS connection attempting to establish inner TLS connection %s", logConn(conn))
+			conn.Close()
+			return
+		}
 		conn = tls.Server(conn, s.tlsConfigurator.IncomingInsecureRPCConfig())
 		s.handleInsecureConn(conn)
 
 	default:
 		if !s.handleEnterpriseRPCConn(typ, conn, isTLS) {
-			s.logger.Printf("[ERR] consul.rpc: unrecognized RPC byte: %v %s", typ, logConn(conn))
+			s.rpcLogger().Error("unrecognized RPC byte",
+				"byte", typ,
+				"conn", logConn(conn),
+			)
 			conn.Close()
 		}
 	}
@@ -132,7 +183,10 @@ func (s *Server) handleMultiplexV2(conn net.Conn) {
 		sub, err := server.Accept()
 		if err != nil {
 			if err != io.EOF {
-				s.logger.Printf("[ERR] consul.rpc: multiplex conn accept failed: %v %s", err, logConn(conn))
+				s.rpcLogger().Error("multiplex conn accept failed",
+					"conn", logConn(conn),
+					"error", err,
+				)
 			}
 			return
 		}
@@ -143,7 +197,7 @@ func (s *Server) handleMultiplexV2(conn net.Conn) {
 // handleConsulConn is used to service a single Consul RPC connection
 func (s *Server) handleConsulConn(conn net.Conn) {
 	defer conn.Close()
-	rpcCodec := msgpackrpc.NewServerCodec(conn)
+	rpcCodec := msgpackrpc.NewCodecFromHandle(true, true, conn, structs.MsgpackHandle)
 	for {
 		select {
 		case <-s.shutdownCh:
@@ -153,7 +207,10 @@ func (s *Server) handleConsulConn(conn net.Conn) {
 
 		if err := s.rpcServer.ServeRequest(rpcCodec); err != nil {
 			if err != io.EOF && !strings.Contains(err.Error(), "closed") {
-				s.logger.Printf("[ERR] consul.rpc: RPC error: %v %s", err, logConn(conn))
+				s.rpcLogger().Error("RPC error",
+					"conn", logConn(conn),
+					"error", err,
+				)
 				metrics.IncrCounter([]string{"rpc", "request_error"}, 1)
 			}
 			return
@@ -165,7 +222,7 @@ func (s *Server) handleConsulConn(conn net.Conn) {
 // handleInsecureConsulConn is used to service a single Consul INSECURERPC connection
 func (s *Server) handleInsecureConn(conn net.Conn) {
 	defer conn.Close()
-	rpcCodec := msgpackrpc.NewServerCodec(conn)
+	rpcCodec := msgpackrpc.NewCodecFromHandle(true, true, conn, structs.MsgpackHandle)
 	for {
 		select {
 		case <-s.shutdownCh:
@@ -175,7 +232,10 @@ func (s *Server) handleInsecureConn(conn net.Conn) {
 
 		if err := s.insecureRPCServer.ServeRequest(rpcCodec); err != nil {
 			if err != io.EOF && !strings.Contains(err.Error(), "closed") {
-				s.logger.Printf("[ERR] consul.rpc: INSECURERPC error: %v %s", err, logConn(conn))
+				s.rpcLogger().Error("INSECURERPC error",
+					"conn", logConn(conn),
+					"error", err,
+				)
 				metrics.IncrCounter([]string{"rpc", "request_error"}, 1)
 			}
 			return
@@ -190,7 +250,10 @@ func (s *Server) handleSnapshotConn(conn net.Conn) {
 	go func() {
 		defer conn.Close()
 		if err := s.handleSnapshotRequest(conn); err != nil {
-			s.logger.Printf("[ERR] consul.rpc: Snapshot RPC error: %v %s", err, logConn(conn))
+			s.rpcLogger().Error("Snapshot RPC error",
+				"conn", logConn(conn),
+				"error", err,
+			)
 		}
 	}()
 }
@@ -203,6 +266,12 @@ func canRetry(args interface{}, err error) bool {
 		return true
 	}
 
+	// If we are chunking and it doesn't seem to have completed, try again
+	intErr, ok := args.(error)
+	if ok && strings.Contains(intErr.Error(), ErrChunkingResubmit.Error()) {
+		return true
+	}
+
 	// Reads are safe to retry for stream errors, such as if a server was
 	// being shut down.
 	info, ok := args.(structs.RPCInfo)
@@ -302,7 +371,14 @@ func (s *Server) getLeader() (bool, *metadata.Server) {
 func (s *Server) forwardDC(method, dc string, args interface{}, reply interface{}) error {
 	manager, server, ok := s.router.FindRoute(dc)
 	if !ok {
-		s.logger.Printf("[WARN] consul.rpc: RPC request for DC %q, no path found", dc)
+		if s.router.HasDatacenter(dc) {
+			s.rpcLogger().Warn("RPC request to DC is currently failing as no server can be reached", "datacenter", dc)
+			return structs.ErrDCNotAvailable
+		}
+		s.rpcLogger().Warn("RPC request for DC is currently failing as no path was found",
+			"datacenter", dc,
+			"method", method,
+		)
 		return structs.ErrNoDCPath
 	}
 
@@ -310,7 +386,12 @@ func (s *Server) forwardDC(method, dc string, args interface{}, reply interface{
 		[]metrics.Label{{Name: "datacenter", Value: dc}})
 	if err := s.connPool.RPC(dc, server.Addr, server.Version, method, server.UseTLS, args, reply); err != nil {
 		manager.NotifyFailedServer(server)
-		s.logger.Printf("[ERR] consul: RPC failed to server %s in DC %q: %v", server.Addr, dc, err)
+		s.rpcLogger().Error("RPC failed to server in DC",
+			"server", server.Addr,
+			"datacenter", dc,
+			"method", method,
+			"error", err,
+		)
 		return err
 	}
 
@@ -353,25 +434,80 @@ func (s *Server) globalRPC(method string, args interface{},
 	return nil
 }
 
+type raftEncoder func(structs.MessageType, interface{}) ([]byte, error)
+
 // raftApply is used to encode a message, run it through raft, and return
 // the FSM response along with any errors
 func (s *Server) raftApply(t structs.MessageType, msg interface{}) (interface{}, error) {
-	buf, err := structs.Encode(t, msg)
+	return s.raftApplyMsgpack(t, msg)
+}
+
+// raftApplyMsgpack will msgpack encode the request and then run it through raft,
+// then return the FSM response along with any errors.
+func (s *Server) raftApplyMsgpack(t structs.MessageType, msg interface{}) (interface{}, error) {
+	return s.raftApplyWithEncoder(t, msg, structs.Encode)
+}
+
+// raftApplyProtobuf will protobuf encode the request and then run it through raft,
+// then return the FSM response along with any errors.
+func (s *Server) raftApplyProtobuf(t structs.MessageType, msg interface{}) (interface{}, error) {
+	return s.raftApplyWithEncoder(t, msg, structs.EncodeProtoInterface)
+}
+
+// raftApplyWithEncoder is used to encode a message, run it through raft,
+// and return the FSM response along with any errors. Unlike raftApply this
+// takes the encoder to use as an argument.
+func (s *Server) raftApplyWithEncoder(t structs.MessageType, msg interface{}, encoder raftEncoder) (interface{}, error) {
+	if encoder == nil {
+		return nil, fmt.Errorf("Failed to encode request: nil encoder")
+	}
+	buf, err := encoder(t, msg)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to encode request: %v", err)
 	}
 
 	// Warn if the command is very large
 	if n := len(buf); n > raftWarnSize {
-		s.logger.Printf("[WARN] consul: Attempting to apply large raft entry (%d bytes)", n)
+		s.rpcLogger().Warn("Attempting to apply large raft entry", "size_in_bytes", n)
+	}
+
+	var chunked bool
+	var future raft.ApplyFuture
+	switch {
+	case len(buf) <= raft.SuggestedMaxDataSize || t != structs.KVSRequestType:
+		future = s.raft.Apply(buf, enqueueLimit)
+	default:
+		chunked = true
+		future = raftchunking.ChunkingApply(buf, nil, enqueueLimit, s.raft.ApplyLog)
 	}
 
-	future := s.raft.Apply(buf, enqueueLimit)
 	if err := future.Error(); err != nil {
 		return nil, err
 	}
 
-	return future.Response(), nil
+	resp := future.Response()
+
+	if chunked {
+		// In this case we didn't apply all chunks successfully, possibly due
+		// to a term change; resubmit
+		if resp == nil {
+			// This returns the error in the interface because the raft library
+			// returns errors from the FSM via the future, not via err from the
+			// apply function. Downstream client code expects to see any error
+			// from the FSM (as opposed to the apply itself) and decide whether
+			// it can retry in the future's response.
+			return ErrChunkingResubmit, nil
+		}
+		// We expect that this conversion should always work
+		chunkedSuccess, ok := resp.(raftchunking.ChunkingSuccess)
+		if !ok {
+			return nil, errors.New("unknown type of response back from chunking FSM")
+		}
+		// Return the inner wrapped response
+		return chunkedSuccess.Response, nil
+	}
+
+	return resp, nil
 }
 
 // queryFn is used to perform a query operation. If a re-query is needed, the
@@ -382,42 +518,57 @@ func (s *Server) raftApply(t structs.MessageType, msg interface{}) (interface{},
 type queryFn func(memdb.WatchSet, *state.Store) error
 
 // blockingQuery is used to process a potentially blocking query operation.
-func (s *Server) blockingQuery(queryOpts *structs.QueryOptions, queryMeta *structs.QueryMeta,
-	fn queryFn) error {
+func (s *Server) blockingQuery(queryOpts structs.QueryOptionsCompat, queryMeta structs.QueryMetaCompat, fn queryFn) error {
 	var timeout *time.Timer
+	var queriesBlocking uint64
+	var queryTimeout time.Duration
 
+	// Instrument all queries run
+	metrics.IncrCounter([]string{"rpc", "query"}, 1)
+
+	minQueryIndex := queryOpts.GetMinQueryIndex()
 	// Fast path right to the non-blocking query.
-	if queryOpts.MinQueryIndex == 0 {
+	if minQueryIndex == 0 {
 		goto RUN_QUERY
 	}
 
+	queryTimeout = queryOpts.GetMaxQueryTime()
 	// Restrict the max query time, and ensure there is always one.
-	if queryOpts.MaxQueryTime > maxQueryTime {
-		queryOpts.MaxQueryTime = maxQueryTime
-	} else if queryOpts.MaxQueryTime <= 0 {
-		queryOpts.MaxQueryTime = defaultQueryTime
+	if queryTimeout > s.config.MaxQueryTime {
+		queryTimeout = s.config.MaxQueryTime
+	} else if queryTimeout <= 0 {
+		queryTimeout = s.config.DefaultQueryTime
 	}
 
 	// Apply a small amount of jitter to the request.
-	queryOpts.MaxQueryTime += lib.RandomStagger(queryOpts.MaxQueryTime / jitterFraction)
+	queryTimeout += lib.RandomStagger(queryTimeout / jitterFraction)
 
 	// Setup a query timeout.
-	timeout = time.NewTimer(queryOpts.MaxQueryTime)
+	timeout = time.NewTimer(queryTimeout)
 	defer timeout.Stop()
 
+	// instrument blockingQueries
+	// atomic inc our server's count of in-flight blockingQueries and store the new value
+	queriesBlocking = atomic.AddUint64(&s.queriesBlocking, 1)
+	// atomic dec when we return from blockingQuery()
+	defer atomic.AddUint64(&s.queriesBlocking, ^uint64(0))
+	// set the gauge directly to the new value of s.blockingQueries
+	metrics.SetGauge([]string{"rpc", "queries_blocking"}, float32(queriesBlocking))
+
 RUN_QUERY:
+	// Setup blocking loop
 	// Update the query metadata.
 	s.setQueryMeta(queryMeta)
 
+	// Validate
 	// If the read must be consistent we verify that we are still the leader.
-	if queryOpts.RequireConsistent {
+	if queryOpts.GetRequireConsistent() {
 		if err := s.consistentRead(); err != nil {
 			return err
 		}
 	}
 
-	// Run the query.
-	metrics.IncrCounter([]string{"rpc", "query"}, 1)
+	// Run query
 
 	// Operate on a consistent set of state. This makes sure that the
 	// abandon channel goes with the state that the caller is using to
@@ -426,7 +577,7 @@ RUN_QUERY:
 
 	// We can skip all watch tracking if this isn't a blocking query.
 	var ws memdb.WatchSet
-	if queryOpts.MinQueryIndex > 0 {
+	if minQueryIndex > 0 {
 		ws = memdb.NewWatchSet()
 
 		// This channel will be closed if a snapshot is restored and the
@@ -434,7 +585,7 @@ RUN_QUERY:
 		ws.Add(state.AbandonCh())
 	}
 
-	// Block up to the timeout if we didn't see anything fresh.
+	// Execute the queryFn
 	err := fn(ws, state)
 	// Note we check queryOpts.MinQueryIndex is greater than zero to determine if
 	// blocking was requested by client, NOT meta.Index since the state function
@@ -444,10 +595,11 @@ RUN_QUERY:
 	// state method). We also need to ensure that unless there is an error, we
 	// return an index > 0 otherwise the client will never block and burn CPU and
 	// requests.
-	if err == nil && queryMeta.Index < 1 {
-		queryMeta.Index = 1
+	if err == nil && queryMeta.GetIndex() < 1 {
+		queryMeta.SetIndex(1)
 	}
-	if err == nil && queryOpts.MinQueryIndex > 0 && queryMeta.Index <= queryOpts.MinQueryIndex {
+	// block up to the timeout if we don't see anything fresh.
+	if err == nil && minQueryIndex > 0 && queryMeta.GetIndex() <= minQueryIndex {
 		if expired := ws.Watch(timeout.C); !expired {
 			// If a restore may have woken us up then bail out from
 			// the query immediately. This is slightly race-ey since
@@ -457,6 +609,7 @@ RUN_QUERY:
 			select {
 			case <-state.AbandonCh():
 			default:
+				// loop back and look for an update again
 				goto RUN_QUERY
 			}
 		}
@@ -465,13 +618,13 @@ RUN_QUERY:
 }
 
 // setQueryMeta is used to populate the QueryMeta data for an RPC call
-func (s *Server) setQueryMeta(m *structs.QueryMeta) {
+func (s *Server) setQueryMeta(m structs.QueryMetaCompat) {
 	if s.IsLeader() {
-		m.LastContact = 0
-		m.KnownLeader = true
+		m.SetLastContact(0)
+		m.SetKnownLeader(true)
 	} else {
-		m.LastContact = time.Since(s.raft.LastContact())
-		m.KnownLeader = (s.raft.Leader() != "")
+		m.SetLastContact(time.Since(s.raft.LastContact()))
+		m.SetKnownLeader(s.raft.Leader() != "")
 	}
 }
 
