@@ -1,24 +1,25 @@
 package consul
 
 import (
-	"context"
 	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
-	"log"
 	"net"
 	"net/rpc"
 	"os"
 	"path/filepath"
 	"reflect"
 	"strconv"
+	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
 
 	metrics "github.com/armon/go-metrics"
+	"github.com/hashicorp/consul/acl"
 	ca "github.com/hashicorp/consul/agent/connect/ca"
+	"github.com/hashicorp/consul/agent/consul/authmethod"
 	"github.com/hashicorp/consul/agent/consul/autopilot"
 	"github.com/hashicorp/consul/agent/consul/fsm"
 	"github.com/hashicorp/consul/agent/consul/state"
@@ -28,9 +29,10 @@ import (
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/lib"
-	"github.com/hashicorp/consul/sentinel"
+	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/tlsutil"
 	"github.com/hashicorp/consul/types"
+	connlimit "github.com/hashicorp/go-connlimit"
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/raft"
@@ -87,6 +89,20 @@ const (
 	reconcileChSize = 256
 )
 
+const (
+	legacyACLReplicationRoutineName    = "legacy ACL replication"
+	aclPolicyReplicationRoutineName    = "ACL policy replication"
+	aclRoleReplicationRoutineName      = "ACL role replication"
+	aclTokenReplicationRoutineName     = "ACL token replication"
+	aclTokenReapingRoutineName         = "acl token reaping"
+	aclUpgradeRoutineName              = "legacy ACL token upgrade"
+	caRootPruningRoutineName           = "CA root pruning"
+	configReplicationRoutineName       = "config entry replication"
+	intentionReplicationRoutineName    = "intention replication"
+	secondaryCARootWatchRoutineName    = "secondary CA roots watch"
+	secondaryCertRenewWatchRoutineName = "secondary cert renew watch"
+)
+
 var (
 	ErrWANFederationDisabled = fmt.Errorf("WAN Federation is disabled")
 )
@@ -94,32 +110,19 @@ var (
 // Server is Consul server which manages the service discovery,
 // health checking, DC forwarding, Raft, and multiple Serf pools.
 type Server struct {
-	// sentinel is the Sentinel code engine (can be nil).
-	sentinel sentinel.Evaluator
+	// queriesBlocking is a counter that we incr and decr atomically in
+	// rpc calls to provide telemetry on how many blocking queries are running.
+	// We interact with queriesBlocking atomically, do not move without ensuring it is
+	// correctly 64-byte aligned in the struct layout
+	queriesBlocking uint64
+
+	// aclConfig is the configuration for the ACL system
+	aclConfig *acl.Config
 
 	// acls is used to resolve tokens to effective policies
 	acls *ACLResolver
 
-	// aclUpgradeCancel is used to cancel the ACL upgrade goroutine when we
-	// lose leadership
-	aclUpgradeCancel  context.CancelFunc
-	aclUpgradeLock    sync.RWMutex
-	aclUpgradeEnabled bool
-
-	// aclReplicationCancel is used to shut down the ACL replication goroutine
-	// when we lose leadership
-	aclReplicationCancel  context.CancelFunc
-	aclReplicationLock    sync.RWMutex
-	aclReplicationEnabled bool
-
-	// aclTokenReapCancel is used to shut down the ACL Token expiration reap
-	// goroutine when we lose leadership.
-	aclTokenReapCancel  context.CancelFunc
-	aclTokenReapLock    sync.RWMutex
-	aclTokenReapEnabled bool
-
-	aclAuthMethodValidators    map[string]*authMethodValidatorEntry
-	aclAuthMethodValidatorLock sync.RWMutex
+	aclAuthMethodValidators authmethod.Cache
 
 	// DEPRECATED (ACL-Legacy-Compat) - only needed while we support both
 	// useNewACLs is used to determine whether we can use new ACLs or not
@@ -131,6 +134,8 @@ type Server struct {
 	// autopilotWaitGroup is used to block until Autopilot shuts down.
 	autopilotWaitGroup sync.WaitGroup
 
+	// caProviderReconfigurationLock guards the provider reconfiguration.
+	caProviderReconfigurationLock sync.Mutex
 	// caProvider is the current CA provider in use for Connect. This is
 	// only non-nil when we are the leader.
 	caProvider ca.Provider
@@ -141,12 +146,6 @@ type Server struct {
 	caProviderRoot *structs.CARoot
 	caProviderLock sync.RWMutex
 
-	// caPruningCh is used to shut down the CA root pruning goroutine when we
-	// lose leadership.
-	caPruningCh      chan struct{}
-	caPruningLock    sync.RWMutex
-	caPruningEnabled bool
-
 	// Consul configuration
 	config *Config
 
@@ -175,7 +174,8 @@ type Server struct {
 	fsm *fsm.FSM
 
 	// Logger uses the provided LogOutput
-	logger *log.Logger
+	logger  hclog.InterceptLogger
+	loggers *loggerStore
 
 	// The raft instance is used among Consul nodes within the DC to protect
 	// operations that require strong consistency.
@@ -213,6 +213,9 @@ type Server struct {
 	// from an agent.
 	rpcLimiter atomic.Value
 
+	// rpcConnLimiter limits the number of RPC connections from a single source IP
+	rpcConnLimiter connlimit.Limiter
+
 	// Listener is used to listen for incoming connections
 	Listener  net.Listener
 	rpcServer *rpc.Server
@@ -276,6 +279,13 @@ type Server struct {
 	shutdownCh   chan struct{}
 	shutdownLock sync.Mutex
 
+	// State for whether this datacenter is acting as a secondary CA.
+	actingSecondaryCA   bool
+	actingSecondaryLock sync.RWMutex
+
+	// Manager to handle starting/stopping go routines when establishing/revoking raft leadership
+	leaderRoutineManager *LeaderRoutineManager
+
 	// embedded struct to hold all the enterprise specific data
 	EnterpriseServer
 }
@@ -292,7 +302,7 @@ func NewServer(config *Config) (*Server, error) {
 
 // NewServerLogger is used to construct a new Consul server from the
 // configuration, potentially returning an error
-func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tlsConfigurator *tlsutil.Configurator) (*Server, error) {
+func NewServerLogger(config *Config, logger hclog.InterceptLogger, tokens *token.Store, tlsConfigurator *tlsutil.Configurator) (*Server, error) {
 	// Check the protocol version.
 	if err := config.CheckProtocolVersion(); err != nil {
 		return nil, err
@@ -312,8 +322,12 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 	if config.LogOutput == nil {
 		config.LogOutput = os.Stderr
 	}
+
 	if logger == nil {
-		logger = log.New(config.LogOutput, "", log.LstdFlags)
+		logger = hclog.NewInterceptLogger(&hclog.LoggerOptions{
+			Level:  hclog.Debug,
+			Output: config.LogOutput,
+		})
 	}
 
 	// Check if TLS is enabled
@@ -330,6 +344,10 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 		}
 	}
 
+	if config.PrimaryDatacenter != "" {
+		config.ACLDatacenter = config.PrimaryDatacenter
+	}
+
 	// Create the tombstone GC.
 	gc, err := state.NewTombstoneGC(config.TombstoneTTL, config.TombstoneTTLGranularity)
 	if err != nil {
@@ -348,26 +366,31 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 		ForceTLS:        config.VerifyOutgoing,
 	}
 
+	serverLogger := logger.NamedIntercept(logging.ConsulServer)
+	loggers := newLoggerStore(serverLogger)
 	// Create server.
 	s := &Server{
-		config:            config,
-		tokens:            tokens,
-		connPool:          connPool,
-		eventChLAN:        make(chan serf.Event, serfEventChSize),
-		eventChWAN:        make(chan serf.Event, serfEventChSize),
-		logger:            logger,
-		leaveCh:           make(chan struct{}),
-		reconcileCh:       make(chan serf.Member, reconcileChSize),
-		router:            router.NewRouter(logger, config.Datacenter),
-		rpcServer:         rpc.NewServer(),
-		insecureRPCServer: rpc.NewServer(),
-		tlsConfigurator:   tlsConfigurator,
-		reassertLeaderCh:  make(chan chan error),
-		segmentLAN:        make(map[string]*serf.Serf, len(config.Segments)),
-		sessionTimers:     NewSessionTimers(),
-		tombstoneGC:       gc,
-		serverLookup:      NewServerLookup(),
-		shutdownCh:        shutdownCh,
+		config:                  config,
+		tokens:                  tokens,
+		connPool:                connPool,
+		eventChLAN:              make(chan serf.Event, serfEventChSize),
+		eventChWAN:              make(chan serf.Event, serfEventChSize),
+		logger:                  serverLogger,
+		loggers:                 loggers,
+		leaveCh:                 make(chan struct{}),
+		reconcileCh:             make(chan serf.Member, reconcileChSize),
+		router:                  router.NewRouter(serverLogger, config.Datacenter),
+		rpcServer:               rpc.NewServer(),
+		insecureRPCServer:       rpc.NewServer(),
+		tlsConfigurator:         tlsConfigurator,
+		reassertLeaderCh:        make(chan chan error),
+		segmentLAN:              make(map[string]*serf.Serf, len(config.Segments)),
+		sessionTimers:           NewSessionTimers(),
+		tombstoneGC:             gc,
+		serverLookup:            NewServerLookup(),
+		shutdownCh:              shutdownCh,
+		leaderRoutineManager:    NewLeaderRoutineManager(logger),
+		aclAuthMethodValidators: authmethod.NewCache(),
 	}
 
 	// Initialize enterprise specific server functionality
@@ -379,11 +402,11 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 	s.rpcLimiter.Store(rate.NewLimiter(config.RPCRate, config.RPCMaxBurst))
 
 	configReplicatorConfig := ReplicatorConfig{
-		Name:        "Config Entry",
-		ReplicateFn: s.replicateConfig,
-		Rate:        s.config.ConfigReplicationRate,
-		Burst:       s.config.ConfigReplicationBurst,
-		Logger:      logger,
+		Name:     logging.ConfigEntry,
+		Delegate: &FunctionReplicator{ReplicateFn: s.replicateConfig},
+		Rate:     s.config.ConfigReplicationRate,
+		Burst:    s.config.ConfigReplicationBurst,
+		Logger:   s.logger,
 	}
 	s.configReplicator, err = NewReplicator(&configReplicatorConfig)
 	if err != nil {
@@ -394,7 +417,7 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 	// Initialize the stats fetcher that autopilot will use.
 	s.statsFetcher = NewStatsFetcher(logger, s.connPool, s.config.Datacenter)
 
-	s.sentinel = sentinel.New(logger)
+	s.aclConfig = newACLConfig(logger)
 	s.useNewACLs = 0
 	aclConfig := ACLResolverConfig{
 		Config:      config,
@@ -402,7 +425,7 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 		CacheConfig: serverACLCacheConfig,
 		AutoDisable: false,
 		Logger:      logger,
-		Sentinel:    s.sentinel,
+		ACLConfig:   s.aclConfig,
 	}
 	// Initialize the ACL resolver.
 	if s.acls, err = NewACLResolver(&aclConfig); err != nil {
@@ -457,7 +480,7 @@ func NewServerLogger(config *Config, logger *log.Logger, tokens *token.Store, tl
 			if serfBindPortWAN == 0 {
 				return nil, fmt.Errorf("Failed to get dynamic bind port for WAN Serf")
 			}
-			s.logger.Printf("[INFO] agent: Serf WAN TCP bound to port %d", serfBindPortWAN)
+			s.logger.Info("Serf WAN TCP bound", "port", serfBindPortWAN)
 		}
 	}
 
@@ -529,7 +552,7 @@ func (s *Server) trackAutoEncryptCARoots() {
 	for {
 		select {
 		case <-s.shutdownCh:
-			s.logger.Printf("[DEBUG] agent: shutting down trackAutoEncryptCARoots because shutdown")
+			s.logger.Debug("shutting down trackAutoEncryptCARoots because shutdown")
 			return
 		default:
 		}
@@ -538,7 +561,7 @@ func (s *Server) trackAutoEncryptCARoots() {
 		ws.Add(state.AbandonCh())
 		_, cas, err := state.CARoots(ws)
 		if err != nil {
-			s.logger.Printf("[DEBUG] agent: Failed to watch AutoEncrypt CARoot: %v", err)
+			s.logger.Error("Failed to watch AutoEncrypt CARoot", "error", err)
 			return
 		}
 		caPems := []string{}
@@ -546,7 +569,7 @@ func (s *Server) trackAutoEncryptCARoots() {
 			caPems = append(caPems, ca.RootCert)
 		}
 		if err := s.tlsConfigurator.UpdateAutoEncryptCA(caPems); err != nil {
-			s.logger.Printf("[DEBUG] agent: Failed to update AutoEncrypt CAPems: %v", err)
+			s.logger.Error("Failed to update AutoEncrypt CAPems", "error", err)
 		}
 		ws.Watch(nil)
 	}
@@ -558,14 +581,14 @@ func (s *Server) setupRaft() error {
 	defer func() {
 		if s.raft == nil && s.raftStore != nil {
 			if err := s.raftStore.Close(); err != nil {
-				s.logger.Printf("[ERR] consul: failed to close Raft store: %v", err)
+				s.logger.Error("failed to close Raft store", "error", err)
 			}
 		}
 	}()
 
 	// Create the FSM.
 	var err error
-	s.fsm, err = fsm.New(s.tombstoneGC, s.config.LogOutput)
+	s.fsm, err = fsm.New(s.tombstoneGC, s.logger)
 	if err != nil {
 		return err
 	}
@@ -581,21 +604,12 @@ func (s *Server) setupRaft() error {
 		MaxPool:               3,
 		Timeout:               10 * time.Second,
 		ServerAddressProvider: serverAddressProvider,
-		Logger:                s.logger,
+		Logger:                s.loggers.Named(logging.Raft),
 	}
 
 	trans := raft.NewNetworkTransportWithConfig(transConfig)
 	s.raftTransport = trans
-
-	// Make sure we set the LogOutput.
-	s.config.RaftConfig.LogOutput = s.config.LogOutput
-	raftLogger := hclog.New(&hclog.LoggerOptions{
-		Name:       "raft",
-		Level:      hclog.LevelFromString(s.config.LogLevel),
-		Output:     s.config.LogOutput,
-		TimeFormat: `2006/01/02 15:04:05`,
-	})
-	s.config.RaftConfig.Logger = raftLogger
+	s.config.RaftConfig.Logger = s.loggers.Named(logging.Raft)
 
 	// Versions of the Raft protocol below 3 require the LocalID to match the network
 	// address of the transport.
@@ -664,10 +678,10 @@ func (s *Server) setupRaft() error {
 				if err := os.Remove(peersFile); err != nil {
 					return fmt.Errorf("failed to delete peers.json, please delete manually (see peers.info for details): %v", err)
 				}
-				s.logger.Printf("[INFO] consul: deleted peers.json file (see peers.info for details)")
+				s.logger.Info("deleted peers.json file (see peers.info for details)")
 			}
 		} else if _, err := os.Stat(peersFile); err == nil {
-			s.logger.Printf("[INFO] consul: found peers.json file, recovering Raft configuration...")
+			s.logger.Info("found peers.json file, recovering Raft configuration...")
 
 			var configuration raft.Configuration
 			if s.config.RaftConfig.ProtocolVersion < 3 {
@@ -679,7 +693,7 @@ func (s *Server) setupRaft() error {
 				return fmt.Errorf("recovery failed to parse peers.json: %v", err)
 			}
 
-			tmpFsm, err := fsm.New(s.tombstoneGC, s.config.LogOutput)
+			tmpFsm, err := fsm.New(s.tombstoneGC, s.logger)
 			if err != nil {
 				return fmt.Errorf("recovery failed to make temp FSM: %v", err)
 			}
@@ -691,7 +705,7 @@ func (s *Server) setupRaft() error {
 			if err := os.Remove(peersFile); err != nil {
 				return fmt.Errorf("recovery failed to delete peers.json, please delete manually (see peers.info for details): %v", err)
 			}
-			s.logger.Printf("[INFO] consul: deleted peers.json file after successful recovery")
+			s.logger.Info("deleted peers.json file after successful recovery")
 		}
 	}
 
@@ -719,12 +733,12 @@ func (s *Server) setupRaft() error {
 	}
 
 	// Set up a channel for reliable leader notifications.
-	raftNotifyCh := make(chan bool, 1)
+	raftNotifyCh := make(chan bool, 10)
 	s.config.RaftConfig.NotifyCh = raftNotifyCh
 	s.raftNotifyCh = raftNotifyCh
 
 	// Setup the Raft store.
-	s.raft, err = raft.NewRaft(s.config.RaftConfig, s.fsm, log, stable, snap, trans)
+	s.raft, err = raft.NewRaft(s.config.RaftConfig, s.fsm.ChunkingFSM(), log, stable, snap, trans)
 	if err != nil {
 		return err
 	}
@@ -745,6 +759,10 @@ func registerEndpoint(fn factory) {
 
 // setupRPC is used to setup the RPC listener
 func (s *Server) setupRPC() error {
+	s.rpcConnLimiter.SetConfig(connlimit.Config{
+		MaxConnsPerClientIP: s.config.RPCMaxConnsPerClient,
+	})
+
 	for _, fn := range endpoints {
 		s.rpcServer.Register(fn(s))
 	}
@@ -797,7 +815,7 @@ func (s *Server) setupRPC() error {
 
 // Shutdown is used to shutdown the server
 func (s *Server) Shutdown() error {
-	s.logger.Printf("[INFO] consul: shutting down server")
+	s.logger.Info("shutting down server")
 	s.shutdownLock.Lock()
 	defer s.shutdownLock.Unlock()
 
@@ -808,6 +826,11 @@ func (s *Server) Shutdown() error {
 	s.shutdown = true
 	close(s.shutdownCh)
 
+	// ensure that any leader routines still running get canceled
+	if s.leaderRoutineManager != nil {
+		s.leaderRoutineManager.StopAll()
+	}
+
 	if s.serfLAN != nil {
 		s.serfLAN.Shutdown()
 	}
@@ -815,7 +838,7 @@ func (s *Server) Shutdown() error {
 	if s.serfWAN != nil {
 		s.serfWAN.Shutdown()
 		if err := s.router.RemoveArea(types.AreaWAN); err != nil {
-			s.logger.Printf("[WARN] consul: error removing WAN area: %v", err)
+			s.logger.Warn("error removing WAN area", "error", err)
 		}
 	}
 	s.router.Shutdown()
@@ -825,7 +848,7 @@ func (s *Server) Shutdown() error {
 		s.raftLayer.Close()
 		future := s.raft.Shutdown()
 		if err := future.Error(); err != nil {
-			s.logger.Printf("[WARN] consul: error shutting down raft: %s", err)
+			s.logger.Warn("error shutting down raft", "error", err)
 		}
 		if s.raftStore != nil {
 			s.raftStore.Close()
@@ -839,17 +862,23 @@ func (s *Server) Shutdown() error {
 	// Close the connection pool
 	s.connPool.Shutdown()
 
+	s.acls.Close()
+
+	if s.config.NotifyShutdown != nil {
+		s.config.NotifyShutdown()
+	}
+
 	return nil
 }
 
 // Leave is used to prepare for a graceful shutdown of the server
 func (s *Server) Leave() error {
-	s.logger.Printf("[INFO] consul: server starting leave")
+	s.logger.Info("server starting leave")
 
 	// Check the number of known peers
 	numPeers, err := s.numPeers()
 	if err != nil {
-		s.logger.Printf("[ERR] consul: failed to check raft peers: %v", err)
+		s.logger.Error("failed to check raft peers", "error", err)
 		return err
 	}
 
@@ -869,12 +898,12 @@ func (s *Server) Leave() error {
 		if minRaftProtocol >= 2 && s.config.RaftConfig.ProtocolVersion >= 3 {
 			future := s.raft.RemoveServer(raft.ServerID(s.config.NodeID), 0, 0)
 			if err := future.Error(); err != nil {
-				s.logger.Printf("[ERR] consul: failed to remove ourself as raft peer: %v", err)
+				s.logger.Error("failed to remove ourself as raft peer", "error", err)
 			}
 		} else {
 			future := s.raft.RemovePeer(addr)
 			if err := future.Error(); err != nil {
-				s.logger.Printf("[ERR] consul: failed to remove ourself as raft peer: %v", err)
+				s.logger.Error("failed to remove ourself as raft peer", "error", err)
 			}
 		}
 	}
@@ -882,22 +911,25 @@ func (s *Server) Leave() error {
 	// Leave the WAN pool
 	if s.serfWAN != nil {
 		if err := s.serfWAN.Leave(); err != nil {
-			s.logger.Printf("[ERR] consul: failed to leave WAN Serf cluster: %v", err)
+			s.logger.Error("failed to leave WAN Serf cluster", "error", err)
 		}
 	}
 
 	// Leave the LAN pool
 	if s.serfLAN != nil {
 		if err := s.serfLAN.Leave(); err != nil {
-			s.logger.Printf("[ERR] consul: failed to leave LAN Serf cluster: %v", err)
+			s.logger.Error("failed to leave LAN Serf cluster", "error", err)
 		}
 	}
 
+	// Leave everything enterprise related as well
+	s.handleEnterpriseLeave()
+
 	// Start refusing RPCs now that we've left the LAN pool. It's important
 	// to do this *after* we've left the LAN pool so that clients will know
 	// to shift onto another server if they perform a retry. We also wake up
 	// all queries in the RPC retry state.
-	s.logger.Printf("[INFO] consul: Waiting %s to drain RPC traffic", s.config.LeaveDrainTime)
+	s.logger.Info("Waiting to drain RPC traffic", "drain_time", s.config.LeaveDrainTime)
 	close(s.leaveCh)
 	time.Sleep(s.config.LeaveDrainTime)
 
@@ -914,7 +946,7 @@ func (s *Server) Leave() error {
 			// Get the latest configuration.
 			future := s.raft.GetConfiguration()
 			if err := future.Error(); err != nil {
-				s.logger.Printf("[ERR] consul: failed to get raft configuration: %v", err)
+				s.logger.Error("failed to get raft configuration", "error", err)
 				break
 			}
 
@@ -944,7 +976,7 @@ func (s *Server) Leave() error {
 		// may not realize that it has been removed. Need to revisit this
 		// and the warning here.
 		if !left {
-			s.logger.Printf("[WARN] consul: failed to leave raft configuration gracefully, timeout")
+			s.logger.Warn("failed to leave raft configuration gracefully, timeout")
 		}
 	}
 
@@ -999,12 +1031,24 @@ func (s *Server) WANMembers() []serf.Member {
 }
 
 // RemoveFailedNode is used to remove a failed node from the cluster
-func (s *Server) RemoveFailedNode(node string) error {
-	if err := s.serfLAN.RemoveFailedNode(node); err != nil {
+func (s *Server) RemoveFailedNode(node string, prune bool) error {
+	var removeFn func(*serf.Serf, string) error
+	if prune {
+		removeFn = (*serf.Serf).RemoveFailedNodePrune
+	} else {
+		removeFn = (*serf.Serf).RemoveFailedNode
+	}
+
+	if err := removeFn(s.serfLAN, node); err != nil {
 		return err
 	}
+	// The Serf WAN pool stores members as node.datacenter
+	// so the dc is appended if not present
+	if !strings.HasSuffix(node, "."+s.config.Datacenter) {
+		node = node + "." + s.config.Datacenter
+	}
 	if s.serfWAN != nil {
-		if err := s.serfWAN.RemoveFailedNode(node); err != nil {
+		if err := removeFn(s.serfWAN, node); err != nil {
 			return err
 		}
 	}
@@ -1133,7 +1177,7 @@ func (s *Server) SnapshotRPC(args *structs.SnapshotRequest, in io.Reader, out io
 	}
 	defer func() {
 		if err := snap.Close(); err != nil {
-			s.logger.Printf("[ERR] consul: Failed to close snapshot: %v", err)
+			s.logger.Error("Failed to close snapshot", "error", err)
 		}
 	}()
 
@@ -1155,7 +1199,7 @@ func (s *Server) SnapshotRPC(args *structs.SnapshotRequest, in io.Reader, out io
 
 // RegisterEndpoint is used to substitute an endpoint for testing.
 func (s *Server) RegisterEndpoint(name string, handler interface{}) error {
-	s.logger.Printf("[WARN] consul: endpoint injected; this should only be used for testing")
+	s.logger.Warn("endpoint injected; this should only be used for testing")
 	return s.rpcServer.RegisterName(name, handler)
 }
 
@@ -1228,6 +1272,9 @@ func (s *Server) GetLANCoordinate() (lib.CoordinateSet, error) {
 // relevant configuration information
 func (s *Server) ReloadConfig(config *Config) error {
 	s.rpcLimiter.Store(rate.NewLimiter(config.RPCRate, config.RPCMaxBurst))
+	s.rpcConnLimiter.SetConfig(connlimit.Config{
+		MaxConnsPerClientIP: config.RPCMaxConnsPerClient,
+	})
 
 	if s.IsLeader() {
 		// only bootstrap the config entries if we are the leader
@@ -1252,6 +1299,10 @@ func (s *Server) isReadyForConsistentReads() bool {
 	return atomic.LoadInt32(&s.readyForConsistentReads) == 1
 }
 
+func (s *Server) intentionReplicationEnabled() bool {
+	return s.config.ConnectEnabled && s.config.Datacenter != s.config.PrimaryDatacenter
+}
+
 // peersInfoContent is used to help operators understand what happened to the
 // peers.json file. This is written to a file called peers.info in the same
 // location.
