@@ -8,13 +8,15 @@ import (
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/go-uuid"
 )
 
 // Session endpoint is used to manipulate sessions for KV
 type Session struct {
-	srv *Server
+	srv    *Server
+	logger hclog.Logger
 }
 
 // Apply is used to apply a modifying request to the data store. This should
@@ -33,28 +35,37 @@ func (s *Session) Apply(args *structs.SessionRequest, reply *string) error {
 		return fmt.Errorf("Must provide Node")
 	}
 
+	//  The entMeta to populate is the one in the Session struct, not SessionRequest
+	//  This is because the Session is what is passed to downstream functions like raftApply
+	var authzContext acl.AuthorizerContext
+
 	// Fetch the ACL token, if any, and apply the policy.
-	rule, err := s.srv.ResolveToken(args.Token)
+	authz, err := s.srv.ResolveTokenAndDefaultMeta(args.Token, &args.Session.EnterpriseMeta, &authzContext)
 	if err != nil {
 		return err
 	}
-	if rule != nil && s.srv.config.ACLEnforceVersion8 {
+
+	if err := s.srv.validateEnterpriseRequest(&args.Session.EnterpriseMeta, true); err != nil {
+		return err
+	}
+
+	if authz != nil && s.srv.config.ACLEnforceVersion8 {
 		switch args.Op {
 		case structs.SessionDestroy:
 			state := s.srv.fsm.State()
-			_, existing, err := state.SessionGet(nil, args.Session.ID)
+			_, existing, err := state.SessionGet(nil, args.Session.ID, &args.Session.EnterpriseMeta)
 			if err != nil {
 				return fmt.Errorf("Session lookup failed: %v", err)
 			}
 			if existing == nil {
 				return fmt.Errorf("Unknown session %q", args.Session.ID)
 			}
-			if !rule.SessionWrite(existing.Node) {
+			if authz.SessionWrite(existing.Node, &authzContext) != acl.Allow {
 				return acl.ErrPermissionDenied
 			}
 
 		case structs.SessionCreate:
-			if !rule.SessionWrite(args.Session.Node) {
+			if authz.SessionWrite(args.Session.Node, &authzContext) != acl.Allow {
 				return acl.ErrPermissionDenied
 			}
 
@@ -97,12 +108,12 @@ func (s *Session) Apply(args *structs.SessionRequest, reply *string) error {
 		for {
 			var err error
 			if args.Session.ID, err = uuid.GenerateUUID(); err != nil {
-				s.srv.logger.Printf("[ERR] consul.session: UUID generation failed: %v", err)
+				s.logger.Error("UUID generation failed", "error", err)
 				return err
 			}
-			_, sess, err := state.SessionGet(nil, args.Session.ID)
+			_, sess, err := state.SessionGet(nil, args.Session.ID, &args.Session.EnterpriseMeta)
 			if err != nil {
-				s.srv.logger.Printf("[ERR] consul.session: Session lookup failed: %v", err)
+				s.logger.Error("Session lookup failed", "error", err)
 				return err
 			}
 			if sess == nil {
@@ -114,7 +125,7 @@ func (s *Session) Apply(args *structs.SessionRequest, reply *string) error {
 	// Apply the update
 	resp, err := s.srv.raftApply(structs.SessionRequestType, args)
 	if err != nil {
-		s.srv.logger.Printf("[ERR] consul.session: Apply failed: %v", err)
+		s.logger.Error("Apply failed", "error", err)
 		return err
 	}
 
@@ -145,11 +156,21 @@ func (s *Session) Get(args *structs.SessionSpecificRequest,
 		return err
 	}
 
+	var authzContext acl.AuthorizerContext
+	authz, err := s.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
+	if err != nil {
+		return err
+	}
+
+	if err := s.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return s.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, session, err := state.SessionGet(ws, args.Session)
+			index, session, err := state.SessionGet(ws, args.SessionID, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
@@ -160,7 +181,7 @@ func (s *Session) Get(args *structs.SessionSpecificRequest,
 			} else {
 				reply.Sessions = nil
 			}
-			if err := s.srv.filterACL(args.Token, reply); err != nil {
+			if err := s.srv.filterACLWithAuthorizer(authz, reply); err != nil {
 				return err
 			}
 			return nil
@@ -168,23 +189,33 @@ func (s *Session) Get(args *structs.SessionSpecificRequest,
 }
 
 // List is used to list all the active sessions
-func (s *Session) List(args *structs.DCSpecificRequest,
+func (s *Session) List(args *structs.SessionSpecificRequest,
 	reply *structs.IndexedSessions) error {
 	if done, err := s.srv.forward("Session.List", args, args, reply); done {
 		return err
 	}
 
+	var authzContext acl.AuthorizerContext
+	authz, err := s.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
+	if err != nil {
+		return err
+	}
+
+	if err := s.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return s.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, sessions, err := state.SessionList(ws)
+			index, sessions, err := state.SessionList(ws, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
 
 			reply.Index, reply.Sessions = index, sessions
-			if err := s.srv.filterACL(args.Token, reply); err != nil {
+			if err := s.srv.filterACLWithAuthorizer(authz, reply); err != nil {
 				return err
 			}
 			return nil
@@ -198,17 +229,27 @@ func (s *Session) NodeSessions(args *structs.NodeSpecificRequest,
 		return err
 	}
 
+	var authzContext acl.AuthorizerContext
+	authz, err := s.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
+	if err != nil {
+		return err
+	}
+
+	if err := s.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return s.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, sessions, err := state.NodeSessions(ws, args.Node)
+			index, sessions, err := state.NodeSessions(ws, args.Node, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
 
 			reply.Index, reply.Sessions = index, sessions
-			if err := s.srv.filterACL(args.Token, reply); err != nil {
+			if err := s.srv.filterACLWithAuthorizer(authz, reply); err != nil {
 				return err
 			}
 			return nil
@@ -223,9 +264,20 @@ func (s *Session) Renew(args *structs.SessionSpecificRequest,
 	}
 	defer metrics.MeasureSince([]string{"session", "renew"}, time.Now())
 
+	// Fetch the ACL token, if any, and apply the policy.
+	var authzContext acl.AuthorizerContext
+	authz, err := s.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
+	if err != nil {
+		return err
+	}
+
+	if err := s.srv.validateEnterpriseRequest(&args.EnterpriseMeta, true); err != nil {
+		return err
+	}
+
 	// Get the session, from local state.
 	state := s.srv.fsm.State()
-	index, session, err := state.SessionGet(nil, args.Session)
+	index, session, err := state.SessionGet(nil, args.SessionID, &args.EnterpriseMeta)
 	if err != nil {
 		return err
 	}
@@ -235,21 +287,14 @@ func (s *Session) Renew(args *structs.SessionSpecificRequest,
 		return nil
 	}
 
-	// Fetch the ACL token, if any, and apply the policy.
-	rule, err := s.srv.ResolveToken(args.Token)
-	if err != nil {
-		return err
-	}
-	if rule != nil && s.srv.config.ACLEnforceVersion8 {
-		if !rule.SessionWrite(session.Node) {
-			return acl.ErrPermissionDenied
-		}
+	if authz != nil && s.srv.config.ACLEnforceVersion8 && authz.SessionWrite(session.Node, &authzContext) != acl.Allow {
+		return acl.ErrPermissionDenied
 	}
 
 	// Reset the session TTL timer.
 	reply.Sessions = structs.Sessions{session}
-	if err := s.srv.resetSessionTimer(args.Session, session); err != nil {
-		s.srv.logger.Printf("[ERR] consul.session: Session renew failed: %v", err)
+	if err := s.srv.resetSessionTimer(args.SessionID, session); err != nil {
+		s.logger.Error("Session renew failed", "error", err)
 		return err
 	}
 
