@@ -11,7 +11,7 @@ import (
 	"time"
 
 	"github.com/hashicorp/consul/acl"
-	"github.com/hashicorp/consul/sentinel"
+	"github.com/hashicorp/consul/lib"
 	"golang.org/x/crypto/blake2b"
 )
 
@@ -80,29 +80,13 @@ service_prefix "" {
 }
 session_prefix "" {
 	policy = "write"
-}`
+}` + EnterpriseACLPolicyGlobalManagement
 
 	// This is the policy ID for anonymous access. This is configurable by the
 	// user.
 	ACLTokenAnonymousID = "00000000-0000-0000-0000-000000000002"
 
 	ACLReservedPrefix = "00000000-0000-0000-0000-0000000000"
-
-	// aclPolicyTemplateServiceIdentity is the template used for synthesizing
-	// policies for service identities.
-	aclPolicyTemplateServiceIdentity = `
-service "%s" {
-	policy = "write"
-}
-service "%s-sidecar-proxy" {
-	policy = "write"
-}
-service_prefix "" {
-	policy = "read"
-}
-node_prefix "" {
-	policy = "read"
-}`
 )
 
 func ACLIDReserved(id string) bool {
@@ -135,6 +119,7 @@ type ACLIdentity interface {
 	EmbeddedPolicy() *ACLPolicy
 	ServiceIdentityList() []*ACLServiceIdentity
 	IsExpired(asOf time.Time) bool
+	EnterpriseMetadata() *EnterpriseMeta
 }
 
 type ACLTokenPolicyLink struct {
@@ -182,10 +167,10 @@ func (s *ACLServiceIdentity) EstimateSize() int {
 	return size
 }
 
-func (s *ACLServiceIdentity) SyntheticPolicy() *ACLPolicy {
+func (s *ACLServiceIdentity) SyntheticPolicy(entMeta *EnterpriseMeta) *ACLPolicy {
 	// Given that we validate this string name before persisting, we do not
 	// have to escape it before doing the following interpolation.
-	rules := fmt.Sprintf(aclPolicyTemplateServiceIdentity, s.ServiceName, s.ServiceName)
+	rules := aclServiceIdentityRules(s.ServiceName, entMeta)
 
 	hasher := fnv.New128a()
 	hashID := fmt.Sprintf("%x", hasher.Sum([]byte(rules)))
@@ -197,6 +182,7 @@ func (s *ACLServiceIdentity) SyntheticPolicy() *ACLPolicy {
 	policy.Rules = rules
 	policy.Syntax = acl.SyntaxCurrent
 	policy.Datacenters = s.Datacenters
+	policy.EnterpriseMeta.Merge(entMeta)
 	policy.SetHash(true)
 	return policy
 }
@@ -242,6 +228,9 @@ type ACLToken struct {
 	// AuthMethod is the name of the auth method used to create this token.
 	AuthMethod string `json:",omitempty"`
 
+	// ACLAuthMethodEnterpriseMeta is the EnterpriseMeta for the AuthMethod that this token was created from
+	ACLAuthMethodEnterpriseMeta
+
 	// ExpirationTime represents the point after which a token should be
 	// considered revoked and is eligible for destruction. The zero value
 	// represents NO expiration.
@@ -269,10 +258,43 @@ type ACLToken struct {
 	// unnecessary calls to the authoritative DC
 	Hash []byte
 
+	// Embedded Enterprise Metadata
+	EnterpriseMeta `mapstructure:",squash"`
+
 	// Embedded Raft Metadata
 	RaftIndex
 }
 
+func (t *ACLToken) UnmarshalJSON(data []byte) (err error) {
+	type Alias ACLToken
+	aux := &struct {
+		ExpirationTTL interface{}
+		Hash          string
+		*Alias
+	}{
+		Alias: (*Alias)(t),
+	}
+
+	if err = lib.UnmarshalJSON(data, &aux); err != nil {
+		return err
+	}
+	if aux.ExpirationTTL != nil {
+		switch v := aux.ExpirationTTL.(type) {
+		case string:
+			if t.ExpirationTTL, err = time.ParseDuration(v); err != nil {
+				return err
+			}
+		case float64:
+			t.ExpirationTTL = time.Duration(v)
+		}
+
+	}
+	if aux.Hash != "" {
+		t.Hash = []byte(aux.Hash)
+	}
+	return nil
+}
+
 func (t *ACLToken) Clone() *ACLToken {
 	t2 := *t
 	t2.Policies = nil
@@ -391,6 +413,10 @@ func (t *ACLToken) EmbeddedPolicy() *ACLPolicy {
 	return policy
 }
 
+func (t *ACLToken) EnterpriseMetadata() *EnterpriseMeta {
+	return &t.EnterpriseMeta
+}
+
 func (t *ACLToken) SetHash(force bool) []byte {
 	if force || t.Hash == nil {
 		// Initialize a 256bit Blake2 hash (32 bytes)
@@ -430,6 +456,8 @@ func (t *ACLToken) SetHash(force bool) []byte {
 			srvid.AddToHash(hash)
 		}
 
+		t.EnterpriseMeta.addToHash(hash, false)
+
 		// Finalize the hash
 		hashVal := hash.Sum(nil)
 
@@ -451,7 +479,7 @@ func (t *ACLToken) EstimateSize() int {
 	for _, srvid := range t.ServiceIdentities {
 		size += srvid.EstimateSize()
 	}
-	return size
+	return size + t.EnterpriseMeta.estimateSize()
 }
 
 // ACLTokens is a slice of ACLTokens.
@@ -471,6 +499,7 @@ type ACLTokenListStub struct {
 	CreateIndex       uint64
 	ModifyIndex       uint64
 	Legacy            bool `json:",omitempty"`
+	EnterpriseMeta
 }
 
 type ACLTokenListStubs []*ACLTokenListStub
@@ -490,6 +519,7 @@ func (token *ACLToken) Stub() *ACLTokenListStub {
 		CreateIndex:       token.CreateIndex,
 		ModifyIndex:       token.ModifyIndex,
 		Legacy:            token.Rules != "",
+		EnterpriseMeta:    token.EnterpriseMeta,
 	}
 }
 
@@ -537,10 +567,31 @@ type ACLPolicy struct {
 	// unnecessary calls to the authoritative DC
 	Hash []byte
 
+	// Embedded Enterprise ACL Metadata
+	EnterpriseMeta `mapstructure:",squash"`
+
 	// Embedded Raft Metadata
 	RaftIndex `hash:"ignore"`
 }
 
+func (t *ACLPolicy) UnmarshalJSON(data []byte) error {
+	type Alias ACLPolicy
+	aux := &struct {
+		Hash string
+		*Alias
+	}{
+		Alias: (*Alias)(t),
+	}
+
+	if err := lib.UnmarshalJSON(data, &aux); err != nil {
+		return err
+	}
+	if aux.Hash != "" {
+		t.Hash = []byte(aux.Hash)
+	}
+	return nil
+}
+
 func (p *ACLPolicy) Clone() *ACLPolicy {
 	p2 := *p
 	p2.Datacenters = cloneStringSlice(p.Datacenters)
@@ -555,17 +606,19 @@ type ACLPolicyListStub struct {
 	Hash        []byte
 	CreateIndex uint64
 	ModifyIndex uint64
+	EnterpriseMeta
 }
 
 func (p *ACLPolicy) Stub() *ACLPolicyListStub {
 	return &ACLPolicyListStub{
-		ID:          p.ID,
-		Name:        p.Name,
-		Description: p.Description,
-		Datacenters: p.Datacenters,
-		Hash:        p.Hash,
-		CreateIndex: p.CreateIndex,
-		ModifyIndex: p.ModifyIndex,
+		ID:             p.ID,
+		Name:           p.Name,
+		Description:    p.Description,
+		Datacenters:    p.Datacenters,
+		Hash:           p.Hash,
+		CreateIndex:    p.CreateIndex,
+		ModifyIndex:    p.ModifyIndex,
+		EnterpriseMeta: p.EnterpriseMeta,
 	}
 }
 
@@ -596,6 +649,8 @@ func (p *ACLPolicy) SetHash(force bool) []byte {
 			hash.Write([]byte(dc))
 		}
 
+		p.EnterpriseMeta.addToHash(hash, false)
+
 		// Finalize the hash
 		hashVal := hash.Sum(nil)
 
@@ -615,7 +670,7 @@ func (p *ACLPolicy) EstimateSize() int {
 		size += len(dc)
 	}
 
-	return size
+	return size + p.EnterpriseMeta.estimateSize()
 }
 
 // HashKey returns a consistent hash for a set of policies.
@@ -645,7 +700,7 @@ func (policies ACLPolicyListStubs) Sort() {
 	})
 }
 
-func (policies ACLPolicies) resolveWithCache(cache *ACLCaches, sentinel sentinel.Evaluator) ([]*acl.Policy, error) {
+func (policies ACLPolicies) resolveWithCache(cache *ACLCaches, entConf *acl.Config) ([]*acl.Policy, error) {
 	// Parse the policies
 	parsed := make([]*acl.Policy, 0, len(policies))
 	for _, policy := range policies {
@@ -658,7 +713,7 @@ func (policies ACLPolicies) resolveWithCache(cache *ACLCaches, sentinel sentinel
 			continue
 		}
 
-		p, err := acl.NewPolicyFromSource(policy.ID, policy.ModifyIndex, policy.Rules, policy.Syntax, sentinel)
+		p, err := acl.NewPolicyFromSource(policy.ID, policy.ModifyIndex, policy.Rules, policy.Syntax, entConf, policy.EnterprisePolicyMeta())
 		if err != nil {
 			return nil, fmt.Errorf("failed to parse %q: %v", policy.Name, err)
 		}
@@ -670,7 +725,7 @@ func (policies ACLPolicies) resolveWithCache(cache *ACLCaches, sentinel sentinel
 	return parsed, nil
 }
 
-func (policies ACLPolicies) Compile(parent acl.Authorizer, cache *ACLCaches, sentinel sentinel.Evaluator) (acl.Authorizer, error) {
+func (policies ACLPolicies) Compile(cache *ACLCaches, entConf *acl.Config) (acl.Authorizer, error) {
 	// Determine the cache key
 	cacheKey := policies.HashKey()
 	entry := cache.GetAuthorizer(cacheKey)
@@ -679,13 +734,13 @@ func (policies ACLPolicies) Compile(parent acl.Authorizer, cache *ACLCaches, sen
 		return entry.Authorizer, nil
 	}
 
-	parsed, err := policies.resolveWithCache(cache, sentinel)
+	parsed, err := policies.resolveWithCache(cache, entConf)
 	if err != nil {
 		return nil, fmt.Errorf("failed to parse the ACL policies: %v", err)
 	}
 
 	// Create the ACL object
-	authorizer, err := acl.NewPolicyAuthorizer(parent, parsed, sentinel)
+	authorizer, err := acl.NewPolicyAuthorizer(parsed, entConf)
 	if err != nil {
 		return nil, fmt.Errorf("failed to construct ACL Authorizer: %v", err)
 	}
@@ -695,8 +750,8 @@ func (policies ACLPolicies) Compile(parent acl.Authorizer, cache *ACLCaches, sen
 	return authorizer, nil
 }
 
-func (policies ACLPolicies) Merge(cache *ACLCaches, sentinel sentinel.Evaluator) (*acl.Policy, error) {
-	parsed, err := policies.resolveWithCache(cache, sentinel)
+func (policies ACLPolicies) Merge(cache *ACLCaches, entConf *acl.Config) (*acl.Policy, error) {
+	parsed, err := policies.resolveWithCache(cache, entConf)
 	if err != nil {
 		return nil, err
 	}
@@ -759,10 +814,31 @@ type ACLRole struct {
 	// unnecessary calls to the authoritative DC
 	Hash []byte
 
+	// Embedded Enterprise ACL metadata
+	EnterpriseMeta `mapstructure:",squash"`
+
 	// Embedded Raft Metadata
 	RaftIndex `hash:"ignore"`
 }
 
+func (t *ACLRole) UnmarshalJSON(data []byte) error {
+	type Alias ACLRole
+	aux := &struct {
+		Hash string
+		*Alias
+	}{
+		Alias: (*Alias)(t),
+	}
+
+	if err := lib.UnmarshalJSON(data, &aux); err != nil {
+		return err
+	}
+	if aux.Hash != "" {
+		t.Hash = []byte(aux.Hash)
+	}
+	return nil
+}
+
 func (r *ACLRole) Clone() *ACLRole {
 	r2 := *r
 	r2.Policies = nil
@@ -807,6 +883,8 @@ func (r *ACLRole) SetHash(force bool) []byte {
 			srvid.AddToHash(hash)
 		}
 
+		r.EnterpriseMeta.addToHash(hash, false)
+
 		// Finalize the hash
 		hashVal := hash.Sum(nil)
 
@@ -829,7 +907,7 @@ func (r *ACLRole) EstimateSize() int {
 		size += srvid.EstimateSize()
 	}
 
-	return size
+	return size + r.EnterpriseMeta.estimateSize()
 }
 
 const (
@@ -889,6 +967,9 @@ type ACLBindingRule struct {
 	// upon the BindType.
 	BindName string
 
+	// Embedded Enterprise ACL metadata
+	EnterpriseMeta `mapstructure:",squash"`
+
 	// Embedded Raft Metadata
 	RaftIndex `hash:"ignore"`
 }
@@ -912,15 +993,17 @@ type ACLAuthMethodListStub struct {
 	Type        string
 	CreateIndex uint64
 	ModifyIndex uint64
+	EnterpriseMeta
 }
 
 func (p *ACLAuthMethod) Stub() *ACLAuthMethodListStub {
 	return &ACLAuthMethodListStub{
-		Name:        p.Name,
-		Description: p.Description,
-		Type:        p.Type,
-		CreateIndex: p.CreateIndex,
-		ModifyIndex: p.ModifyIndex,
+		Name:           p.Name,
+		Description:    p.Description,
+		Type:           p.Type,
+		CreateIndex:    p.CreateIndex,
+		ModifyIndex:    p.ModifyIndex,
+		EnterpriseMeta: p.EnterpriseMeta,
 	}
 }
 
@@ -958,6 +1041,9 @@ type ACLAuthMethod struct {
 	// maps).
 	Config map[string]interface{}
 
+	// Embedded Enterprise ACL Meta
+	EnterpriseMeta `mapstructure:",squash"`
+
 	// Embedded Raft Metadata
 	RaftIndex `hash:"ignore"`
 }
@@ -1018,6 +1104,7 @@ type ACLTokenGetRequest struct {
 	TokenID     string         // id used for the token lookup
 	TokenIDType ACLTokenIDType // The Type of ID used to lookup the token
 	Datacenter  string         // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1029,6 +1116,7 @@ func (r *ACLTokenGetRequest) RequestDatacenter() string {
 type ACLTokenDeleteRequest struct {
 	TokenID    string // ID of the token to delete
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	WriteRequest
 }
 
@@ -1044,6 +1132,8 @@ type ACLTokenListRequest struct {
 	Role          string // Role filter
 	AuthMethod    string // Auth Method filter
 	Datacenter    string // The datacenter to perform the request within
+	ACLAuthMethodEnterpriseMeta
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1114,6 +1204,7 @@ type ACLTokenResponse struct {
 type ACLTokenBatchResponse struct {
 	Tokens   []*ACLToken
 	Redacted bool // whether the token secrets were redacted.
+	Removed  bool // whether any tokens were completely removed
 	QueryMeta
 }
 
@@ -1132,6 +1223,7 @@ func (r *ACLPolicySetRequest) RequestDatacenter() string {
 type ACLPolicyDeleteRequest struct {
 	PolicyID   string // The id of the policy to delete
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	WriteRequest
 }
 
@@ -1143,6 +1235,7 @@ func (r *ACLPolicyDeleteRequest) RequestDatacenter() string {
 type ACLPolicyGetRequest struct {
 	PolicyID   string // id used for the policy lookup
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1153,6 +1246,7 @@ func (r *ACLPolicyGetRequest) RequestDatacenter() string {
 // ACLPolicyListRequest is used at the RPC layer to request a listing of policies
 type ACLPolicyListRequest struct {
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1228,6 +1322,7 @@ func (r *ACLRoleSetRequest) RequestDatacenter() string {
 type ACLRoleDeleteRequest struct {
 	RoleID     string // id of the role to delete
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	WriteRequest
 }
 
@@ -1240,6 +1335,7 @@ type ACLRoleGetRequest struct {
 	RoleID     string // id used for the role lookup (one of RoleID or RoleName is allowed)
 	RoleName   string // name used for the role lookup (one of RoleID or RoleName is allowed)
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1251,6 +1347,7 @@ func (r *ACLRoleGetRequest) RequestDatacenter() string {
 type ACLRoleListRequest struct {
 	Policy     string // Policy filter
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1318,6 +1415,7 @@ func (r *ACLBindingRuleSetRequest) RequestDatacenter() string {
 type ACLBindingRuleDeleteRequest struct {
 	BindingRuleID string // id of the rule to delete
 	Datacenter    string // The datacenter to perform the request within
+	EnterpriseMeta
 	WriteRequest
 }
 
@@ -1329,6 +1427,7 @@ func (r *ACLBindingRuleDeleteRequest) RequestDatacenter() string {
 type ACLBindingRuleGetRequest struct {
 	BindingRuleID string // id used for the rule lookup
 	Datacenter    string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1340,6 +1439,7 @@ func (r *ACLBindingRuleGetRequest) RequestDatacenter() string {
 type ACLBindingRuleListRequest struct {
 	AuthMethod string // optional filter
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1385,6 +1485,7 @@ func (r *ACLAuthMethodSetRequest) RequestDatacenter() string {
 type ACLAuthMethodDeleteRequest struct {
 	AuthMethodName string // name of the auth method to delete
 	Datacenter     string // The datacenter to perform the request within
+	EnterpriseMeta
 	WriteRequest
 }
 
@@ -1396,6 +1497,7 @@ func (r *ACLAuthMethodDeleteRequest) RequestDatacenter() string {
 type ACLAuthMethodGetRequest struct {
 	AuthMethodName string // name used for the auth method lookup
 	Datacenter     string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1406,6 +1508,7 @@ func (r *ACLAuthMethodGetRequest) RequestDatacenter() string {
 // ACLAuthMethodListRequest is used at the RPC layer to request a listing of auth methods
 type ACLAuthMethodListRequest struct {
 	Datacenter string // The datacenter to perform the request within
+	EnterpriseMeta
 	QueryOptions
 }
 
@@ -1434,12 +1537,19 @@ type ACLAuthMethodBatchSetRequest struct {
 // multiple auth method deletions
 type ACLAuthMethodBatchDeleteRequest struct {
 	AuthMethodNames []string
+	// While it may seem odd that AuthMethodNames is associated with a single
+	// EnterpriseMeta, it is okay as this struct is only ever used to
+	// delete a single entry. This is because AuthMethods unlike tokens, policies
+	// and roles are not replicated between datacenters and therefore never
+	// batch applied.
+	EnterpriseMeta
 }
 
 type ACLLoginParams struct {
 	AuthMethod  string
 	BearerToken string
 	Meta        map[string]string `json:",omitempty"`
+	EnterpriseMeta
 }
 
 type ACLLoginRequest struct {
@@ -1460,3 +1570,43 @@ type ACLLogoutRequest struct {
 func (r *ACLLogoutRequest) RequestDatacenter() string {
 	return r.Datacenter
 }
+
+type RemoteACLAuthorizationRequest struct {
+	Datacenter string
+	Requests   []ACLAuthorizationRequest
+	QueryOptions
+}
+
+type ACLAuthorizationRequest struct {
+	Resource acl.Resource
+	Segment  string `json:",omitempty"`
+	Access   string
+	EnterpriseMeta
+}
+
+type ACLAuthorizationResponse struct {
+	ACLAuthorizationRequest
+	Allow bool
+}
+
+func (r *RemoteACLAuthorizationRequest) RequestDatacenter() string {
+	return r.Datacenter
+}
+
+func CreateACLAuthorizationResponses(authz acl.Authorizer, requests []ACLAuthorizationRequest) ([]ACLAuthorizationResponse, error) {
+	responses := make([]ACLAuthorizationResponse, len(requests))
+	var ctx acl.AuthorizerContext
+
+	for idx, req := range requests {
+		req.FillAuthzContext(&ctx)
+		decision, err := acl.Enforce(authz, req.Resource, req.Segment, req.Access, &ctx)
+		if err != nil {
+			return nil, err
+		}
+
+		responses[idx].ACLAuthorizationRequest = req
+		responses[idx].Allow = decision == acl.Allow
+	}
+
+	return responses, nil
+}
