@@ -5,6 +5,7 @@ import (
 	"reflect"
 	"time"
 
+	"github.com/hashicorp/consul/lib"
 	"github.com/mitchellh/mapstructure"
 )
 
@@ -60,8 +61,8 @@ type CARoot struct {
 	SerialNumber uint64
 
 	// SigningKeyID is the ID of the public key that corresponds to the private
-	// key used to sign the certificate. Is is the HexString format of the raw
-	// AuthorityKeyID bytes.
+	// key used to sign leaf certificates. Is is the HexString format of the
+	// raw AuthorityKeyID bytes.
 	SigningKeyID string
 
 	// ExternalTrustDomain is the trust domain this root was generated under. It
@@ -102,6 +103,16 @@ type CARoot struct {
 	// active root.
 	RotatedOutAt time.Time `json:"-"`
 
+	// PrivateKeyType is the type of the private key used to sign certificates. It
+	// may be "rsa" or "ec". This is provided as a convenience to avoid parsing
+	// the public key to from the certificate to infer the type.
+	PrivateKeyType string
+
+	// PrivateKeyBits is the length of the private key used to sign certificates.
+	// This is provided as a convenience to avoid parsing the public key from the
+	// certificate to infer the type.
+	PrivateKeyBits int
+
 	RaftIndex
 }
 
@@ -140,14 +151,22 @@ type IssuedCert struct {
 
 	// Service is the name of the service for which the cert was issued.
 	// ServiceURI is the cert URI value.
-	Service    string
-	ServiceURI string
+	Service    string `json:",omitempty"`
+	ServiceURI string `json:",omitempty"`
+
+	// Agent is the name of the node for which the cert was issued.
+	// AgentURI is the cert URI value.
+	Agent    string `json:",omitempty"`
+	AgentURI string `json:",omitempty"`
 
 	// ValidAfter and ValidBefore are the validity periods for the
 	// certificate.
 	ValidAfter  time.Time
 	ValidBefore time.Time
 
+	// EnterpriseMeta is the Consul Enterprise specific metadata
+	EnterpriseMeta
+
 	RaftIndex
 }
 
@@ -155,11 +174,12 @@ type IssuedCert struct {
 type CAOp string
 
 const (
-	CAOpSetRoots            CAOp = "set-roots"
-	CAOpSetConfig           CAOp = "set-config"
-	CAOpSetProviderState    CAOp = "set-provider-state"
-	CAOpDeleteProviderState CAOp = "delete-provider-state"
-	CAOpSetRootsAndConfig   CAOp = "set-roots-config"
+	CAOpSetRoots                      CAOp = "set-roots"
+	CAOpSetConfig                     CAOp = "set-config"
+	CAOpSetProviderState              CAOp = "set-provider-state"
+	CAOpDeleteProviderState           CAOp = "delete-provider-state"
+	CAOpSetRootsAndConfig             CAOp = "set-roots-config"
+	CAOpIncrementProviderSerialNumber CAOp = "increment-provider-serial"
 )
 
 // CARequest is used to modify connect CA data. This is used by the
@@ -198,6 +218,7 @@ func (q *CARequest) RequestDatacenter() string {
 const (
 	ConsulCAProvider = "consul"
 	VaultCAProvider  = "vault"
+	AWSCAProvider    = "aws-pca"
 )
 
 // CAConfiguration is the configuration for the current CA plugin.
@@ -213,9 +234,51 @@ type CAConfiguration struct {
 	// and maps).
 	Config map[string]interface{}
 
+	// State is optionally used by the provider to persist information it needs
+	// between reloads like UUIDs of resources it manages. It only supports string
+	// values to avoid gotchas with interface{} since this is encoded through
+	// msgpack when it's written through raft. For example if providers used a
+	// custom struct or even a simple `int` type, msgpack with loose type
+	// information during encode/decode and providers will end up getting back
+	// different types have have to remember to test multiple variants of state
+	// handling to account for cases where it's been through msgpack or not.
+	// Keeping this as strings only forces compatibility and leaves the input
+	// Providers have to work with unambiguous - they can parse ints or other
+	// types as they need. We expect this only to be used to store a handful of
+	// identifiers anyway so this is simpler.
+	State map[string]string
+
+	// ForceWithoutCrossSigning indicates that the CA reconfiguration should go
+	// ahead even if the current CA is unable to cross sign certificates. This
+	// risks temporary connection failures during the rollout as new leafs will be
+	// rejected by proxies that have not yet observed the new root cert but is the
+	// only option if a CA that doesn't support cross signing needs to be
+	// reconfigured or mirated away from.
+	ForceWithoutCrossSigning bool
+
 	RaftIndex
 }
 
+func (c *CAConfiguration) UnmarshalJSON(data []byte) (err error) {
+	type Alias CAConfiguration
+
+	aux := &struct {
+		ForceWithoutCrossSigningSnake bool `json:"force_without_cross_signing"`
+
+		*Alias
+	}{
+		Alias: (*Alias)(c),
+	}
+	if err = lib.UnmarshalJSON(data, &aux); err != nil {
+		return err
+	}
+	if aux.ForceWithoutCrossSigningSnake {
+		c.ForceWithoutCrossSigning = aux.ForceWithoutCrossSigningSnake
+	}
+
+	return nil
+}
+
 func (c *CAConfiguration) GetCommonConfig() (*CommonCAProviderConfig, error) {
 	if c == nil {
 		return nil, fmt.Errorf("config map was nil")
@@ -272,19 +335,49 @@ type CommonCAProviderConfig struct {
 	// immediately in the RPC goroutine. This is 0 by default and CSRMaxPerSecond
 	// is used. This is ignored if CSRMaxPerSecond is non-zero.
 	CSRMaxConcurrent int
+
+	// PrivateKeyType specifies which type of key the CA should generate. It only
+	// applies when the provider is generating its own key and is ignored if the
+	// provider already has a key or an external key is provided. Supported values
+	// are "ec" or "rsa". "ec" is the default and will generate a NIST P-256
+	// Elliptic key.
+	PrivateKeyType string
+
+	// PrivateKeyBits specifies the number of bits the CA's private key should
+	// use. For RSA, supported values are 2048 and 4096. For EC, supported values
+	// are 224, 256, 384 and 521 and correspond to the NIST P-* curve of the same
+	// name. As with PrivateKeyType this is only relevant whan the provier is
+	// generating new CA keys (root or intermediate).
+	PrivateKeyBits int
 }
 
+var MinLeafCertTTL = time.Hour
+var MaxLeafCertTTL = 365 * 24 * time.Hour
+
 func (c CommonCAProviderConfig) Validate() error {
 	if c.SkipValidate {
 		return nil
 	}
 
-	if c.LeafCertTTL < time.Hour {
-		return fmt.Errorf("leaf cert TTL must be greater than 1h")
+	if c.LeafCertTTL < MinLeafCertTTL {
+		return fmt.Errorf("leaf cert TTL must be greater or equal than %s", MinLeafCertTTL)
 	}
 
-	if c.LeafCertTTL > 365*24*time.Hour {
-		return fmt.Errorf("leaf cert TTL must be less than 1 year")
+	if c.LeafCertTTL > MaxLeafCertTTL {
+		return fmt.Errorf("leaf cert TTL must be less than %s", MaxLeafCertTTL)
+	}
+
+	switch c.PrivateKeyType {
+	case "ec":
+		if c.PrivateKeyBits != 224 && c.PrivateKeyBits != 256 && c.PrivateKeyBits != 384 && c.PrivateKeyBits != 521 {
+			return fmt.Errorf("EC key length must be one of (224, 256, 384, 521) bits")
+		}
+	case "rsa":
+		if c.PrivateKeyBits != 2048 && c.PrivateKeyBits != 4096 {
+			return fmt.Errorf("RSA key length must be 2048 or 4096 bits")
+		}
+	default:
+		return fmt.Errorf("private key type must be either 'ec' or 'rsa'")
 	}
 
 	return nil
@@ -293,9 +386,50 @@ func (c CommonCAProviderConfig) Validate() error {
 type ConsulCAProviderConfig struct {
 	CommonCAProviderConfig `mapstructure:",squash"`
 
-	PrivateKey     string
-	RootCert       string
-	RotationPeriod time.Duration
+	PrivateKey          string
+	RootCert            string
+	RotationPeriod      time.Duration
+	IntermediateCertTTL time.Duration
+
+	// DisableCrossSigning is really only useful in test code to use the built in
+	// provider while exercising logic that depends on the CA provider ability to
+	// cross sign. We don't document this config field publicly or make any
+	// attempt to parse it from snake case unlike other fields here.
+	DisableCrossSigning bool
+}
+
+// intermediateCertRenewInterval is the interval at which the expiration
+// of the intermediate cert is checked and renewed if necessary.
+var IntermediateCertRenewInterval = time.Hour
+
+func (c *ConsulCAProviderConfig) Validate() error {
+	if c.IntermediateCertTTL < (3 * IntermediateCertRenewInterval) {
+		// Intermediate Certificates are checked every
+		// hour(intermediateCertRenewInterval) if they are about to
+		// expire. Recreating an intermediate certs is started once
+		// more than half its lifetime has passed.
+		// If it would be 2h, worst case is that the check happens
+		// right before half time and when the check happens again, the
+		// certificate is very close to expiring, leaving only a small
+		// timeframe to renew. 3h leaves more than 30min to recreate.
+		// Right now the minimum LeafCertTTL is 1h, which means this
+		// check not strictly needed, because the same thing is covered
+		// in the next check too. But just in case minimum LeafCertTTL
+		// changes at some point, this validation must still be
+		// performed.
+		return fmt.Errorf("Intermediate Cert TTL must be greater or equal than %dh", 3*int(IntermediateCertRenewInterval.Hours()))
+	}
+	if c.IntermediateCertTTL < (3 * c.CommonCAProviderConfig.LeafCertTTL) {
+		// Intermediate Certificates are being sent to the proxy when
+		// the Leaf Certificate changes because they are bundled
+		// together.
+		// That means that the Intermediate Certificate TTL must be at
+		// a minimum of 3 * Leaf Certificate TTL to ensure that the new
+		// Intermediate is being set together with the Leaf Certificate
+		// before it expires.
+		return fmt.Errorf("Intermediate Cert TTL must be greater or equal than 3 * LeafCertTTL (>=%s).", 3*c.CommonCAProviderConfig.LeafCertTTL)
+	}
+	return nil
 }
 
 // CAConsulProviderState is used to track the built-in Consul CA provider's state.
@@ -324,6 +458,13 @@ type VaultCAProviderConfig struct {
 	TLSSkipVerify bool
 }
 
+type AWSCAProviderConfig struct {
+	CommonCAProviderConfig `mapstructure:",squash"`
+
+	ExistingARN  string
+	DeleteOnExit bool
+}
+
 // CALeafOp is the operation for a request related to leaf certificates.
 type CALeafOp string
 
