@@ -4,6 +4,7 @@ import (
 	"encoding/base64"
 	"fmt"
 	"net/http"
+	"strconv"
 	"strings"
 	"time"
 
@@ -50,40 +51,6 @@ func decodeValue(rawKV interface{}) error {
 	return nil
 }
 
-// fixupTxnOp looks for non-nil Txn operations and passes them on for
-// value conversion.
-func fixupTxnOp(rawOp interface{}) error {
-	rawMap, ok := rawOp.(map[string]interface{})
-	if !ok {
-		return fmt.Errorf("unexpected raw op type: %T", rawOp)
-	}
-	for k, v := range rawMap {
-		switch strings.ToLower(k) {
-		case "kv":
-			if v == nil {
-				return nil
-			}
-			return decodeValue(v)
-		}
-	}
-	return nil
-}
-
-// fixupTxnOps takes the raw decoded JSON and base64 decodes values in Txn ops,
-// replacing them with byte arrays.
-func fixupTxnOps(raw interface{}) error {
-	rawSlice, ok := raw.([]interface{})
-	if !ok {
-		return fmt.Errorf("unexpected raw type: %t", raw)
-	}
-	for _, rawOp := range rawSlice {
-		if err := fixupTxnOp(rawOp); err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
 // isWrite returns true if the given operation alters the state store.
 func isWrite(op api.KVOp) bool {
 	switch op {
@@ -98,11 +65,25 @@ func isWrite(op api.KVOp) bool {
 // a boolean, that if false means an error response has been generated and
 // processing should stop.
 func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (structs.TxnOps, int, bool) {
+
+	sizeStr := req.Header.Get("Content-Length")
+	if sizeStr != "" {
+		if size, err := strconv.Atoi(sizeStr); err != nil {
+			resp.WriteHeader(http.StatusBadRequest)
+			fmt.Fprintf(resp, "Failed to parse Content-Length: %v", err)
+			return nil, 0, false
+		} else if size > int(s.agent.config.KVMaxValueSize) {
+			resp.WriteHeader(http.StatusRequestEntityTooLarge)
+			fmt.Fprintf(resp, "Request body too large, max size: %v bytes", s.agent.config.KVMaxValueSize)
+			return nil, 0, false
+		}
+	}
+
 	// Note the body is in API format, and not the RPC format. If we can't
 	// decode it, we will return a 400 since we don't have enough context to
 	// associate the error with a given operation.
 	var ops api.TxnOps
-	if err := decodeBody(req, &ops, fixupTxnOps); err != nil {
+	if err := decodeBody(req.Body, &ops); err != nil {
 		resp.WriteHeader(http.StatusBadRequest)
 		fmt.Fprintf(resp, "Failed to parse body: %v", err)
 		return nil, 0, false
@@ -123,19 +104,19 @@ func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (st
 	// byte arrays so we can assign right over.
 	var opsRPC structs.TxnOps
 	var writes int
-	var netKVSize int
+	var netKVSize uint64
 	for _, in := range ops {
 		switch {
 		case in.KV != nil:
 			size := len(in.KV.Value)
-			if size > maxKVSize {
+			if uint64(size) > s.agent.config.KVMaxValueSize {
 				resp.WriteHeader(http.StatusRequestEntityTooLarge)
-				fmt.Fprintf(resp, "Value for key %q is too large (%d > %d bytes)", in.KV.Key, size, maxKVSize)
+				fmt.Fprintf(resp, "Value for key %q is too large (%d > %d bytes)", in.KV.Key, size, s.agent.config.KVMaxValueSize)
 				return nil, 0, false
 			}
-			netKVSize += size
+			netKVSize += uint64(size)
 
-			verb := api.KVOp(in.KV.Verb)
+			verb := in.KV.Verb
 			if isWrite(verb) {
 				writes++
 			}
@@ -144,10 +125,11 @@ func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (st
 				KV: &structs.TxnKVOp{
 					Verb: verb,
 					DirEnt: structs.DirEntry{
-						Key:     in.KV.Key,
-						Value:   in.KV.Value,
-						Flags:   in.KV.Flags,
-						Session: in.KV.Session,
+						Key:            in.KV.Key,
+						Value:          in.KV.Value,
+						Flags:          in.KV.Flags,
+						Session:        in.KV.Session,
+						EnterpriseMeta: structs.EnterpriseMetaInitializer(in.KV.Namespace),
 						RaftIndex: structs.RaftIndex{
 							ModifyIndex: in.KV.Index,
 						},
@@ -207,6 +189,7 @@ func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (st
 							Warning: svc.Weights.Warning,
 						},
 						EnableTagOverride: svc.EnableTagOverride,
+						EnterpriseMeta:    structs.EnterpriseMetaInitializer(svc.Namespace),
 						RaftIndex: structs.RaftIndex{
 							ModifyIndex: svc.ModifyIndex,
 						},
@@ -257,11 +240,13 @@ func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (st
 							TLSSkipVerify:                  check.Definition.TLSSkipVerify,
 							Header:                         check.Definition.Header,
 							Method:                         check.Definition.Method,
+							Body:                           check.Definition.Body,
 							TCP:                            check.Definition.TCP,
 							Interval:                       interval,
 							Timeout:                        timeout,
 							DeregisterCriticalServiceAfter: deregisterCriticalServiceAfter,
 						},
+						EnterpriseMeta: structs.EnterpriseMetaInitializer(check.Namespace),
 						RaftIndex: structs.RaftIndex{
 							ModifyIndex: check.ModifyIndex,
 						},
@@ -273,10 +258,10 @@ func (s *HTTPServer) convertOps(resp http.ResponseWriter, req *http.Request) (st
 	}
 
 	// Enforce an overall size limit to help prevent abuse.
-	if netKVSize > maxKVSize {
+	if netKVSize > s.agent.config.KVMaxValueSize {
 		resp.WriteHeader(http.StatusRequestEntityTooLarge)
 		fmt.Fprintf(resp, "Cumulative size of key data is too large (%d > %d bytes)",
-			netKVSize, maxKVSize)
+			netKVSize, s.agent.config.KVMaxValueSize)
 
 		return nil, 0, false
 	}
