@@ -2,19 +2,20 @@ package agent
 
 import (
 	"bytes"
+	"encoding/base64"
 	"fmt"
 	"net/http"
 	"net/http/httptest"
-	"reflect"
+	"strconv"
 	"strings"
 	"testing"
 	"time"
 
+	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/testrpc"
-	"github.com/pascaldekloe/goe/verify"
-
-	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/raft"
+	"github.com/stretchr/testify/assert"
 )
 
 func TestTxnEndpoint_Bad_JSON(t *testing.T) {
@@ -38,10 +39,10 @@ func TestTxnEndpoint_Bad_JSON(t *testing.T) {
 
 func TestTxnEndpoint_Bad_Size_Item(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), "")
-	defer a.Shutdown()
-
-	buf := bytes.NewBuffer([]byte(fmt.Sprintf(`
+	testIt := func(t *testing.T, agent *TestAgent, wantPass bool) {
+		value := strings.Repeat("X", 3*raft.SuggestedMaxDataSize)
+		value = base64.StdEncoding.EncodeToString([]byte(value))
+		buf := bytes.NewBuffer([]byte(fmt.Sprintf(`
  [
      {
          "KV": {
@@ -51,24 +52,41 @@ func TestTxnEndpoint_Bad_Size_Item(t *testing.T) {
          }
      }
  ]
- `, strings.Repeat("bad", 2*maxKVSize))))
-	req, _ := http.NewRequest("PUT", "/v1/txn", buf)
-	resp := httptest.NewRecorder()
-	if _, err := a.srv.Txn(resp, req); err != nil {
-		t.Fatalf("err: %v", err)
-	}
-	if resp.Code != 413 {
-		t.Fatalf("expected 413, got %d", resp.Code)
+ `, value)))
+		req, _ := http.NewRequest("PUT", "/v1/txn", buf)
+		req.Header.Add("Content-Length", fmt.Sprintf("%d", buf.Len()))
+		resp := httptest.NewRecorder()
+		if _, err := agent.srv.Txn(resp, req); err != nil {
+			t.Fatalf("err: %v", err)
+		}
+		if resp.Code != 413 && !wantPass {
+			t.Fatalf("expected 413, got %d", resp.Code)
+		}
+		if resp.Code != 200 && wantPass {
+			t.Fatalf("expected 200, got %d", resp.Code)
+		}
 	}
+
+	t.Run("toobig", func(t *testing.T) {
+		a := NewTestAgent(t, t.Name(), "")
+		testIt(t, a, false)
+		a.Shutdown()
+	})
+
+	t.Run("allowed", func(t *testing.T) {
+		a := NewTestAgent(t, t.Name(), "limits = { kv_max_value_size = 123456789 }")
+		testIt(t, a, true)
+		a.Shutdown()
+	})
 }
 
 func TestTxnEndpoint_Bad_Size_Net(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), "")
-	defer a.Shutdown()
 
-	value := strings.Repeat("X", maxKVSize/2)
-	buf := bytes.NewBuffer([]byte(fmt.Sprintf(`
+	testIt := func(agent *TestAgent, wantPass bool) {
+		value := strings.Repeat("X", 3*raft.SuggestedMaxDataSize)
+		value = base64.StdEncoding.EncodeToString([]byte(value))
+		buf := bytes.NewBuffer([]byte(fmt.Sprintf(`
  [
      {
          "KV": {
@@ -93,14 +111,30 @@ func TestTxnEndpoint_Bad_Size_Net(t *testing.T) {
      }
  ]
  `, value, value, value)))
-	req, _ := http.NewRequest("PUT", "/v1/txn", buf)
-	resp := httptest.NewRecorder()
-	if _, err := a.srv.Txn(resp, req); err != nil {
-		t.Fatalf("err: %v", err)
-	}
-	if resp.Code != 413 {
-		t.Fatalf("expected 413, got %d", resp.Code)
+		req, _ := http.NewRequest("PUT", "/v1/txn", buf)
+		resp := httptest.NewRecorder()
+		if _, err := agent.srv.Txn(resp, req); err != nil {
+			t.Fatalf("err: %v", err)
+		}
+		if resp.Code != 413 && !wantPass {
+			t.Fatalf("expected 413, got %d", resp.Code)
+		}
+		if resp.Code != 200 && wantPass {
+			t.Fatalf("expected 200, got %d", resp.Code)
+		}
 	}
+
+	t.Run("toobig", func(t *testing.T) {
+		a := NewTestAgent(t, t.Name(), "")
+		testIt(a, false)
+		a.Shutdown()
+	})
+
+	t.Run("allowed", func(t *testing.T) {
+		a := NewTestAgent(t, t.Name(), "limits = { kv_max_value_size = 123456789 }")
+		testIt(a, true)
+		a.Shutdown()
+	})
 }
 
 func TestTxnEndpoint_Bad_Size_Ops(t *testing.T) {
@@ -178,7 +212,10 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 			if len(txnResp.Results) != 2 {
 				t.Fatalf("bad: %v", txnResp)
 			}
+
 			index = txnResp.Results[0].KV.ModifyIndex
+			entMeta := txnResp.Results[0].KV.EnterpriseMeta
+
 			expected := structs.TxnResponse{
 				Results: structs.TxnResults{
 					&structs.TxnResult{
@@ -192,6 +229,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 								CreateIndex: index,
 								ModifyIndex: index,
 							},
+							EnterpriseMeta: entMeta,
 						},
 					},
 					&structs.TxnResult{
@@ -205,13 +243,12 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 								CreateIndex: index,
 								ModifyIndex: index,
 							},
+							EnterpriseMeta: entMeta,
 						},
 					},
 				},
 			}
-			if !reflect.DeepEqual(txnResp, expected) {
-				t.Fatalf("bad: %v", txnResp)
-			}
+			assert.Equal(t, expected, txnResp)
 		}
 
 		// Do a read-only transaction that should get routed to the
@@ -256,6 +293,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 			if !ok {
 				t.Fatalf("bad type: %T", obj)
 			}
+			entMeta := txnResp.Results[0].KV.EnterpriseMeta
 			expected := structs.TxnReadResponse{
 				TxnResponse: structs.TxnResponse{
 					Results: structs.TxnResults{
@@ -270,6 +308,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 									CreateIndex: index,
 									ModifyIndex: index,
 								},
+								EnterpriseMeta: entMeta,
 							},
 						},
 						&structs.TxnResult{
@@ -283,6 +322,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 									CreateIndex: index,
 									ModifyIndex: index,
 								},
+								EnterpriseMeta: entMeta,
 							},
 						},
 					},
@@ -291,9 +331,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 					KnownLeader: true,
 				},
 			}
-			if !reflect.DeepEqual(txnResp, expected) {
-				t.Fatalf("bad: %v", txnResp)
-			}
+			assert.Equal(t, expected, txnResp)
 		}
 
 		// Now that we have an index we can do a CAS to make sure the
@@ -334,7 +372,10 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 			if len(txnResp.Results) != 2 {
 				t.Fatalf("bad: %v", txnResp)
 			}
+
 			modIndex := txnResp.Results[0].KV.ModifyIndex
+			entMeta := txnResp.Results[0].KV.EnterpriseMeta
+
 			expected := structs.TxnResponse{
 				Results: structs.TxnResults{
 					&structs.TxnResult{
@@ -346,6 +387,7 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 								CreateIndex: index,
 								ModifyIndex: modIndex,
 							},
+							EnterpriseMeta: entMeta,
 						},
 					},
 					&structs.TxnResult{
@@ -357,13 +399,12 @@ func TestTxnEndpoint_KV_Actions(t *testing.T) {
 								CreateIndex: index,
 								ModifyIndex: modIndex,
 							},
+							EnterpriseMeta: entMeta,
 						},
 					},
 				},
 			}
-			if !reflect.DeepEqual(txnResp, expected) {
-				t.Fatalf("bad: %v", txnResp)
-			}
+			assert.Equal(t, expected, txnResp)
 		}
 	})
 
@@ -520,6 +561,7 @@ func TestTxnEndpoint_UpdateCheck(t *testing.T) {
 						CreateIndex: index,
 						ModifyIndex: index,
 					},
+					EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
 				},
 			},
 			&structs.TxnResult{
@@ -541,6 +583,7 @@ func TestTxnEndpoint_UpdateCheck(t *testing.T) {
 						CreateIndex: index,
 						ModifyIndex: index,
 					},
+					EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
 				},
 			},
 			&structs.TxnResult{
@@ -562,9 +605,57 @@ func TestTxnEndpoint_UpdateCheck(t *testing.T) {
 						CreateIndex: index,
 						ModifyIndex: index,
 					},
+					EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
 				},
 			},
 		},
 	}
-	verify.Values(t, "", txnResp, expected)
+	assert.Equal(t, expected, txnResp)
+}
+
+func TestConvertOps_ContentLength(t *testing.T) {
+	a := NewTestAgent(t, t.Name(), "")
+	defer a.Shutdown()
+
+	jsonBody := `[
+     {
+         "KV": {
+             "Verb": "set",
+             "Key": "key1",
+             "Value": "aGVsbG8gd29ybGQ="
+         }
+     }
+ ]`
+
+	tests := []struct {
+		contentLength string
+		ok            bool
+	}{
+		{"", true},
+		{strconv.Itoa(len(jsonBody)), true},
+		{strconv.Itoa(raft.SuggestedMaxDataSize), true},
+		{strconv.Itoa(raft.SuggestedMaxDataSize + 100), false},
+	}
+
+	for _, tc := range tests {
+		t.Run("contentLength: "+tc.contentLength, func(t *testing.T) {
+			resp := httptest.NewRecorder()
+			var body bytes.Buffer
+
+			// Doesn't matter what the request body size actually is, as we only
+			// check 'Content-Length' header in this test anyway.
+			body.WriteString(jsonBody)
+
+			req := httptest.NewRequest("POST", "http://foo.com", &body)
+			req.Header.Add("Content-Length", tc.contentLength)
+
+			_, _, ok := a.srv.convertOps(resp, req)
+			if ok != tc.ok {
+				t.Fatal("ok != tc.ok")
+			}
+
+		})
+
+	}
+
 }
