@@ -7,7 +7,6 @@ import (
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/logger"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	uuid "github.com/hashicorp/go-uuid"
@@ -41,8 +40,6 @@ func TestBindingRuleUpdateCommand(t *testing.T) {
 		}
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
@@ -444,8 +441,6 @@ func TestBindingRuleUpdateCommand_noMerge(t *testing.T) {
 		}
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
