@@ -7,7 +7,6 @@ import (
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/logger"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
@@ -35,8 +34,6 @@ func TestBootstrapCommand(t *testing.T) {
 		enabled = true
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
