@@ -7,7 +7,6 @@ import (
 	"testing"
 
 	"github.com/hashicorp/consul/agent"
-	"github.com/hashicorp/consul/logger"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
@@ -38,8 +37,6 @@ func TestPolicyCreateCommand(t *testing.T) {
 		}
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
