@@ -7,7 +7,6 @@ import (
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/logger"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
@@ -39,8 +38,6 @@ func TestRoleUpdateCommand(t *testing.T) {
 		}
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
@@ -210,8 +207,6 @@ func TestRoleUpdateCommand_noMerge(t *testing.T) {
 		}
 	}`)
 
-	a.Agent.LogWriter = logger.NewLogWriter(512)
-
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
