@@ -1,10 +1,9 @@
 import Model from 'ember-data/model';
 import attr from 'ember-data/attr';
-import writable from 'consul-ui/utils/model/writable';
 
 export const PRIMARY_KEY = 'uid';
 export const SLUG_KEY = 'ID';
-const model = Model.extend({
+export default Model.extend({
   [PRIMARY_KEY]: attr('string'),
   [SLUG_KEY]: attr('string'),
   Description: attr('string'),
@@ -15,8 +14,11 @@ const model = Model.extend({
   Precedence: attr('number'),
   SourceType: attr('string', { defaultValue: 'consul' }),
   Action: attr('string', { defaultValue: 'deny' }),
+  // These are in the API response but up until now
+  // aren't used for anything
   DefaultAddr: attr('string'),
   DefaultPort: attr('number'),
+  //
   Meta: attr(),
   Datacenter: attr('string'),
   CreatedAt: attr('date'),
@@ -24,11 +26,3 @@ const model = Model.extend({
   CreateIndex: attr('number'),
   ModifyIndex: attr('number'),
 });
-export const ATTRS = writable(model, [
-  'Action',
-  'SourceName',
-  'DestinationName',
-  'SourceType',
-  'Description',
-]);
-export default model;
