@@ -1,7 +1,6 @@
 import Route from '@ember/routing/route';
 import { inject as service } from '@ember/service';
 import { hash } from 'rsvp';
-import { get } from '@ember/object';
 
 import WithRoleActions from 'consul-ui/mixins/role/with-actions';
 
@@ -14,10 +13,12 @@ export default Route.extend(WithRoleActions, {
     },
   },
   model: function(params) {
-    const repo = get(this, 'repo');
     return hash({
-      ...repo.status({
-        items: repo.findAllByDatacenter(this.modelFor('dc').dc.Name),
+      ...this.repo.status({
+        items: this.repo.findAllByDatacenter(
+          this.modelFor('dc').dc.Name,
+          this.modelFor('nspace').nspace.substr(1)
+        ),
       }),
       isLoading: false,
     });
