@@ -5,6 +5,7 @@ import { get } from '@ember/object';
 
 export default Route.extend({
   repo: service('repository/service'),
+  chainRepo: service('repository/discovery-chain'),
   settings: service('settings'),
   queryParams: {
     s: {
@@ -13,13 +14,19 @@ export default Route.extend({
     },
   },
   model: function(params) {
-    const repo = get(this, 'repo');
-    const settings = get(this, 'settings');
     const dc = this.modelFor('dc').dc.Name;
+    const nspace = this.modelFor('nspace').nspace.substr(1);
     return hash({
-      item: repo.findBySlug(params.name, dc),
-      urls: settings.findBySlug('urls'),
+      item: this.repo.findBySlug(params.name, dc, nspace),
+      urls: this.settings.findBySlug('urls'),
       dc: dc,
+    }).then(model => {
+      return hash({
+        chain: ['connect-proxy', 'mesh-gateway'].includes(get(model, 'item.Service.Kind'))
+          ? null
+          : this.chainRepo.findBySlug(params.name, dc, nspace),
+        ...model,
+      });
     });
   },
   setupController: function(controller, model) {
