@@ -1,13 +1,19 @@
 import { moduleFor, test } from 'ember-qunit';
 import repo from 'consul-ui/tests/helpers/repo';
+import { get } from '@ember/object';
 const NAME = 'coordinate';
 moduleFor(`service:repository/${NAME}`, `Integration | Service | ${NAME}`, {
   // Specify the other units that are required for this test.
-  integration: true
+  integration: true,
 });
 
 const dc = 'dc-1';
+const nspace = 'default';
+const now = new Date().getTime();
 test('findAllByDatacenter returns the correct data for list endpoint', function(assert) {
+  get(this.subject(), 'store').serializerFor(NAME).timestamp = function() {
+    return now;
+  };
   return repo(
     'Coordinate',
     'findAllByDatacenter',
@@ -26,8 +32,11 @@ test('findAllByDatacenter returns the correct data for list endpoint', function(
         expected(function(payload) {
           return payload.map(item =>
             Object.assign({}, item, {
+              SyncTime: now,
               Datacenter: dc,
-              uid: `["${dc}","${item.Node}"]`,
+              // TODO: nspace isn't required here, once we've
+              // refactored out our Serializer this can go
+              uid: `["${nspace}","${dc}","${item.Node}"]`,
             })
           );
         })
