@@ -1,7 +1,13 @@
 export default function(visitable, deletable, clickable, attribute, collection, radiogroup) {
   return {
     visit: visitable('/:dc/nodes/:node'),
-    tabs: radiogroup('tab', ['health-checks', 'services', 'round-trip-time', 'lock-sessions']),
+    tabs: radiogroup('tab', [
+      'health-checks',
+      'services',
+      'round-trip-time',
+      'lock-sessions',
+      'meta-data',
+    ]),
     healthchecks: collection('[data-test-node-healthcheck]', {
       name: attribute('data-test-node-healthcheck'),
     }),
@@ -17,5 +23,6 @@ export default function(visitable, deletable, clickable, attribute, collection,
         TTL: attribute('data-test-session-ttl', '[data-test-session-ttl]'),
       })
     ),
+    metaData: collection('#meta-data [data-test-tabular-row]', {}),
   };
 }
